@@ -1,7 +1,7 @@
 #ifndef _ADAPTIVE_VIEWS_H_
 #define _ADAPTIVE_VIEWS_H_
 
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -37,8 +37,8 @@ class GMSH_Post_Plugin;
 class adapt_point
 {
 public :
-  double x, y, z;
-  double X, Y, Z, val;
+  double x,y,z;
+  double X,Y,Z,val,valx,valy,valz;
   double shape_functions[128];
   static adapt_point * New (double x, double y, double z, 
 			    Double_Matrix *coeffs, Double_Matrix *eexps); 
@@ -110,7 +110,7 @@ public:
   }
   inline static void GSF (const double u, const double v, double w, double sf[]) 
   {
-    sf[0] = 1-u-v;
+    sf[0] = 1. - u - v;
     sf[1] = u;
     sf[2] = v;
   }
@@ -149,10 +149,10 @@ public:
   }
   inline static void GSF (const double u, const double v, double w, double sf[]) 
   {
-    sf[0] = 0.25 * (1 - u) * (1 - v);
-    sf[1] = 0.25 * (1 + u) * (1 - v);
-    sf[2] = 0.25 * (1 + u) * (1 + v);
-    sf[3] = 0.25 * (1 - u) * (1 + v);
+    sf[0] = 0.25 * (1. - u) * (1. - v);
+    sf[1] = 0.25 * (1. + u) * (1. - v);
+    sf[2] = 0.25 * (1. + u) * (1. + v);
+    sf[3] = 0.25 * (1. - u) * (1. + v);
   }
   void print ()
   {
@@ -186,7 +186,7 @@ public:
   }
   inline static void GSF (const double u, const double v, double w, double sf[])
   {
-    sf[0] = 1-u-v-w;
+    sf[0] = 1. - u - v - w;
     sf[1] = u;
     sf[2] = v;
     sf[3] = w;
@@ -275,6 +275,10 @@ class Adaptive_Post_View
   Double_Matrix * _STposY;
   Double_Matrix * _STposZ;
   Double_Matrix * _STval;
+  // for vectors
+  Double_Matrix * _STvalX;
+  Double_Matrix * _STvalY;
+  Double_Matrix * _STvalZ;
   Double_Matrix * _Interpolate;
   Double_Matrix * _Geometry;
 public:
