@@ -1,7 +1,7 @@
 #ifndef _COMMAND_LINE_H_
 #define _COMMAND_LINE_H_
 
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,6 +26,18 @@ extern char gmsh_email[], gmsh_gui[], gmsh_options[], gmsh_license[];
 
 void Get_Options(int argc, char *argv[]);
 void Print_Usage(char *name);
-char *Get_BuildOptions(void);
+
+char *Get_BuildOptions();
+
+int Get_GmshMajorVersion();
+int Get_GmshMinorVersion();
+int Get_GmshPatchVersion();
+char *Get_GmshExtraVersion();
+char *Get_GmshVersion();
+char *Get_GmshBuildDate();
+char *Get_GmshBuildHost();
+char *Get_GmshPackager();
+char *Get_GmshBuildOS();
+char *Get_GmshShortLicense();
 
 #endif
