@@ -1,7 +1,7 @@
 #ifndef _OPTIONS_H_
 #define _OPTIONS_H_
 
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -282,12 +282,14 @@ double opt_general_line_width(OPT_ARGS_NUM);
 double opt_general_shine(OPT_ARGS_NUM);
 double opt_general_shine_exponent(OPT_ARGS_NUM);
 double opt_general_color_scheme(OPT_ARGS_NUM);
+double opt_general_background_gradient(OPT_ARGS_NUM);
 double opt_general_verbosity(OPT_ARGS_NUM);
 double opt_general_nopopup(OPT_ARGS_NUM);
 double opt_general_terminal(OPT_ARGS_NUM);
 double opt_general_tooltips(OPT_ARGS_NUM);
 double opt_general_confirm_overwrite(OPT_ARGS_NUM);
 double opt_general_orthographic(OPT_ARGS_NUM);
+double opt_general_mouse_selection(OPT_ARGS_NUM);
 double opt_general_draw_bounding_box(OPT_ARGS_NUM);
 double opt_general_fast_redraw(OPT_ARGS_NUM);
 double opt_general_axes(OPT_ARGS_NUM);
@@ -315,6 +317,7 @@ double opt_general_arrow_stem_radius(OPT_ARGS_NUM);
 double opt_general_trackball(OPT_ARGS_NUM);
 double opt_general_rotation_center_cg(OPT_ARGS_NUM);
 double opt_general_zoom_factor(OPT_ARGS_NUM);
+double opt_general_expert_mode(OPT_ARGS_NUM);
 double opt_general_clip0(OPT_ARGS_NUM);
 double opt_general_clip0a(OPT_ARGS_NUM);
 double opt_general_clip0b(OPT_ARGS_NUM);
@@ -398,9 +401,11 @@ double opt_geometry_light(OPT_ARGS_NUM);
 double opt_geometry_old_circle(OPT_ARGS_NUM);
 double opt_geometry_old_newreg(OPT_ARGS_NUM);
 double opt_geometry_circle_points(OPT_ARGS_NUM);
-double opt_geometry_circle_warning(OPT_ARGS_NUM);
 double opt_geometry_extrude_spline_points(OPT_ARGS_NUM);
 double opt_geometry_scaling_factor(OPT_ARGS_NUM);
+double opt_geometry_snap0(OPT_ARGS_NUM);
+double opt_geometry_snap1(OPT_ARGS_NUM);
+double opt_geometry_snap2(OPT_ARGS_NUM);
 double opt_mesh_optimize(OPT_ARGS_NUM);
 double opt_mesh_quality(OPT_ARGS_NUM);
 double opt_mesh_normals(OPT_ARGS_NUM);
@@ -437,6 +442,7 @@ double opt_mesh_light_lines(OPT_ARGS_NUM);
 double opt_mesh_light_two_side(OPT_ARGS_NUM);
 double opt_mesh_format(OPT_ARGS_NUM);
 double opt_mesh_msh_file_version(OPT_ARGS_NUM);
+double opt_mesh_renumber_nodes_continuous(OPT_ARGS_NUM);
 double opt_mesh_nb_smoothing(OPT_ARGS_NUM);
 double opt_mesh_stl_distance_tol(OPT_ARGS_NUM);
 double opt_mesh_nb_elem_per_rc(OPT_ARGS_NUM);
@@ -455,6 +461,7 @@ double opt_mesh_min_circ_points(OPT_ARGS_NUM);
 double opt_mesh_constrained_bgmesh(OPT_ARGS_NUM);
 double opt_mesh_order(OPT_ARGS_NUM);
 double opt_mesh_dual(OPT_ARGS_NUM);
+double opt_mesh_initial_only(OPT_ARGS_NUM);
 double opt_mesh_interactive(OPT_ARGS_NUM);
 double opt_mesh_use_cut_plane(OPT_ARGS_NUM);
 double opt_mesh_cut_plane_as_surface(OPT_ARGS_NUM);
@@ -475,6 +482,7 @@ double opt_mesh_nb_prisms(OPT_ARGS_NUM);
 double opt_mesh_nb_pyramids(OPT_ARGS_NUM);
 double opt_mesh_cpu_time(OPT_ARGS_NUM);
 double opt_solver_max_delay(OPT_ARGS_NUM);
+double opt_solver_listen(OPT_ARGS_NUM);
 double opt_solver_plugins(OPT_ARGS_NUM);
 double opt_solver_client_server(OPT_ARGS_NUM);
 double opt_solver_client_server0(OPT_ARGS_NUM);
@@ -627,6 +635,7 @@ double opt_print_gif_transparent(OPT_ARGS_NUM);
 // COLORS
 
 unsigned int opt_general_color_background(OPT_ARGS_COL);
+unsigned int opt_general_color_background_gradient(OPT_ARGS_COL);
 unsigned int opt_general_color_foreground(OPT_ARGS_COL);
 unsigned int opt_general_color_text(OPT_ARGS_COL);
 unsigned int opt_general_color_axes(OPT_ARGS_COL);
@@ -716,14 +725,17 @@ typedef struct {
   int level;
   char *str ; 
   unsigned int (*function)(int num, int action, unsigned int val) ;
-  unsigned int def1, def2, def3 ;
+  // the defaults are stored in individual bytes so that we can initialize
+  // them statically independently of the machine endianness. They will be
+  // packed into unsigned ints at runtime
+  unsigned char def1[4], def2[4], def3[4] ;
   char *help ;
 } StringXColor ;
 
-void Init_Options (int num);
-void Init_Options_GUI (int num);
-void ReInit_Options (int num);
-void Print_Options(int num, int level, int diff, char *filename);
+void Init_Options(int num);
+void Init_Options_GUI(int num);
+void ReInit_Options(int num);
+void Print_Options(int num, int level, int diff, int help, char *filename);
 void Print_OptionsDoc();
 
 StringXString * Get_StringOptionCategory(char * cat);
@@ -742,12 +754,12 @@ void * Get_StringOption(char *str, StringXString s[]);
 void * Get_NumberOption(char *str, StringXNumber s[]);
 void * Get_ColorOption(char *str, StringXColor s[]);
 
-void Print_StringOptions(int num, int level, int diff, StringXString s[], 
-			 char *prefix, FILE *file);
-void Print_NumberOptions(int num, int level, int diff, StringXNumber s[], 
-			 char *prefix, FILE *file);
-void Print_ColorOptions(int num, int level, int diff, StringXColor s[], 
-			char *prefix, FILE *file);
+void Print_StringOptions(int num, int level, int diff, int help,
+			 StringXString s[], char *prefix, FILE *file);
+void Print_NumberOptions(int num, int level, int diff, int help,
+			 StringXNumber s[], char *prefix, FILE *file);
+void Print_ColorOptions(int num, int level, int diff, int help,
+			StringXColor s[], char *prefix, FILE *file);
 
 void Print_StringOptionsDoc(StringXString s[], char *prefix, FILE * file);
 void Print_NumberOptionsDoc(StringXNumber s[], char *prefix, FILE * file);
