@@ -1,6 +1,6 @@
-// $Id: SafeIO.cpp,v 1.5 2005/01/01 19:35:27 geuzaine Exp $
+// $Id: SafeIO.cpp,v 1.8 2006/02/26 16:33:50 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -22,8 +22,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <unistd.h>
-
 #include "SafeIO.h"
 #include "Message.h"
 
@@ -70,11 +68,8 @@ int safe_fwrite(const void *ptr, size_t size, size_t nmemb, FILE * stream)
       Msg(GERROR, "Disk full");
     else
       Msg(GERROR, strerror(errno));
-    Msg(GERROR, "Truncating output file");
     if(fflush(stream) < 0)
       Msg(GERROR, "EOF reached");
-    if(ftruncate(fileno(stream), 0) < 0)
-      Msg(GERROR, strerror(errno));
     if(fclose(stream) < 0)
       Msg(GERROR, strerror(errno));
     return 1;
