@@ -1,7 +1,7 @@
 #ifndef _CALLBACKS_H_
 #define _CALLBACKS_H_
 
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -118,6 +118,7 @@ void view_save_binary_cb(CALLBACK_ARGS);
 void view_save_parsed_cb(CALLBACK_ARGS);
 void view_save_stl_cb(CALLBACK_ARGS);
 void view_save_txt_cb(CALLBACK_ARGS);
+void view_save_msh_cb(CALLBACK_ARGS);
 void view_alias_cb(CALLBACK_ARGS);
 void view_alias_with_options_cb(CALLBACK_ARGS);
 void view_combine_space_all_cb(CALLBACK_ARGS);
@@ -262,11 +263,11 @@ void geometry_reload_cb(CALLBACK_ARGS);
 
 void con_geometry_define_parameter_cb(CALLBACK_ARGS);
 void con_geometry_define_point_cb(CALLBACK_ARGS);
+void con_geometry_snap_cb(CALLBACK_ARGS);
 
 // Dynamic Mesh Menus
 
 void mesh_save_cb(CALLBACK_ARGS);
-void mesh_save_all_cb(CALLBACK_ARGS);
 void mesh_define_cb(CALLBACK_ARGS);
 void mesh_1d_cb(CALLBACK_ARGS);
 void mesh_2d_cb(CALLBACK_ARGS); 
