@@ -1,7 +1,7 @@
 #ifndef _GUI_H_
 #define _GUI_H_
 
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -33,7 +33,6 @@
 #include <FL/Fl_Return_Button.H>
 #include <FL/Fl_Repeat_Button.H>
 #include <FL/Fl_Light_Button.H>
-#include <FL/Fl_Menu_Button.H>
 #include <FL/Fl_Check_Button.H>
 #include <FL/Fl_Input.H>
 #include <FL/Fl_Value_Input.H>
@@ -47,6 +46,8 @@
 #include <FL/Fl_Color_Chooser.H>
 #include <FL/fl_ask.H>
 #include <FL/Fl_Tooltip.H>
+#include <FL/Fl_Shared_Image.H>
+#include <FL/Fl_File_Icon.H>
 #if defined(__APPLE__) && defined(HAVE_FLTK_1_1_5_OR_ABOVE)
 #include <FL/Fl_Sys_Menu_Bar.H>
 #endif
@@ -56,6 +57,7 @@
 
 #include "Opengl_Window.h"
 #include "Colorbar_Window.h"
+#include "Popup_Button.h"
 
 // The dynamic contexts
 
@@ -126,14 +128,12 @@ class GUI{
   Fl_Scroll *m_scroll;
 
   // Bitmaps
-  Fl_Bitmap  *abort_bmp, *start_bmp, *stop_bmp, *rewind_bmp ;
-  void add_post_plugins ( Fl_Menu_Button *button , int iView);
+  Fl_Bitmap  *abort_bmp, *start_bmp, *stop_bmp, *rewind_bmp, *rotate_bmp, *ortho_bmp ;
+  void add_post_plugins(Popup_Button *button , int iView);
   void add_multiline_in_browser(Fl_Browser *o, char* prefix, char *str);
 
 public:
 
-  Fl_Bitmap  *ortho_bmp, *persp_bmp;
-
   // menu window
   Fl_Window        *m_window ;
 #if defined(__APPLE__) && defined(HAVE_FLTK_1_1_5_OR_ABOVE)
@@ -145,15 +145,15 @@ public:
   std::vector<Fl_Button*>       m_push_butt ;
   std::vector<Fl_Light_Button*> m_toggle_butt ;
   std::vector<Fl_Button*>       m_toggle2_butt ;
-  std::vector<Fl_Menu_Button*>  m_popup_butt ;
-  std::vector<Fl_Menu_Button*>  m_popup2_butt ;
+  std::vector<Popup_Button*>    m_popup_butt ;
+  std::vector<Popup_Button*>    m_popup2_butt ;
   std::vector<char*>            m_pop_label ;
   std::vector<std::pair<int, GMSH_Plugin*>*> m_pop_plugin;
 
   // graphic window
   Fl_Window        *g_window ;
   Opengl_Window    *g_opengl_window ;
-  Fl_Button        *g_status_butt[8] ;
+  Fl_Button        *g_status_butt[10] ;
   Fl_Box           *g_status_label[3] ;
 
   // Option window
@@ -243,6 +243,7 @@ public:
   // geometry context window
   Fl_Window        *context_geometry_window ;
   Fl_Input         *context_geometry_input[30] ;
+  Fl_Value_Input   *context_geometry_value[10] ;
 
   // mesh context window
   Fl_Window        *context_mesh_window ;
@@ -274,7 +275,7 @@ public:
   void create_clip_window();
   void create_manip_window();
   void create_statistics_window();
-  void create_message_window();
+  void create_message_window(bool redraw_only=false);
   void create_about_window();
   void create_geometry_context_window(int num);
   void call_for_solver_plugin (int dim);
@@ -309,7 +310,7 @@ public:
   void update_manip_window(int force=0);
   void reset_external_view_list();
   int  selection, try_selection, quit_selection, end_selection, undo_selection;
-
+  int  try_selection_xywh[4];
 };
 
 // some utility font functions
