@@ -1,6 +1,6 @@
-// $Id: GUI_Extras.cpp,v 1.11 2005/06/10 20:59:15 geuzaine Exp $
+// $Id: GUI_Extras.cpp,v 1.14.2.1 2006/04/24 00:28:49 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,10 +27,12 @@
 #include "CreateFile.h"
 #include "Options.h"
 #include "Context.h"
+#include "Draw.h"
 
 #include <FL/Fl_Button.H>
 #include <FL/Fl_Return_Button.H>
 #include <FL/Fl_Value_Slider.H>
+#include <FL/Fl_Menu_Window.H>
 #include <errno.h>
 
 extern Context_T CTX;
@@ -163,6 +165,55 @@ int arrow_editor(char *title, double &a, double &b, double &c)
   return 0;
 }
 
+// Perspective editor (aka z-clipping planes factor slider)
+
+static void persp_change_factor(Fl_Widget* w, void* data){
+  opt_general_clip_factor(0, GMSH_SET|GMSH_GUI, ((Fl_Slider*)w)->value());
+  Draw();
+}
+
+class Release_Slider : public Fl_Slider {
+  int handle(int event){ 
+    switch (event) {
+    case FL_RELEASE: 
+      if(window())
+	window()->hide();
+      return 1;
+    default:
+      return Fl_Slider::handle(event);
+    }
+  };
+public:
+  Release_Slider(int x,int y,int w,int h,const char *l=0)
+    : Fl_Slider(x, y, w, h, l) {}
+};
+
+int perspective_editor()
+{
+  struct _editor{
+    Fl_Menu_Window *window;
+    Release_Slider *sa;
+  };
+  static _editor *editor = NULL;
+
+  if(!editor){
+    editor = new _editor;
+    editor->window = new Fl_Menu_Window(20, 100);
+    editor->sa = new Release_Slider(0, 0, 20, 100);
+    editor->sa->type(FL_VERT_NICE_SLIDER);
+    editor->sa->minimum(12);
+    editor->sa->maximum(0.75);
+    editor->sa->callback(persp_change_factor);
+    editor->window->border(0);
+    editor->window->end();
+  }
+
+  editor->window->hotspot(editor->window);
+  editor->sa->value(CTX.clip_factor);
+  editor->window->show();
+  return 0;
+}
+
 // jpeg dialog
 
 int jpeg_dialog(char *name, int TeX)
@@ -402,22 +453,27 @@ int options_dialog(char *name)
 {
   struct _options_dialog{
     Fl_Window *window;
-    Fl_Check_Button *b;
+    Fl_Check_Button *b[2];
     Fl_Button *ok, *cancel;
   };
   static _options_dialog *dialog = NULL;
 
   if(!dialog){
     dialog = new _options_dialog;
-    int h = 3*10 + 25 + 1*25, y = 0;
+    int h = 3*10 + 25 + 2*25, y = 0;
     // not a "Dialog_Window" since it is modal 
     dialog->window = new Fl_Double_Window(200, h, "Options"); y = 10;
     dialog->window->box(GMSH_WINDOW_BOX);
-    dialog->b = new Fl_Check_Button(10, y, 180, 25, "Save only modified options"); y += 25;
-    dialog->b->value(1);
-    dialog->b->type(FL_TOGGLE_BUTTON);
-    dialog->b->down_box(GMSH_TOGGLE_BOX);
-    dialog->b->selection_color(GMSH_TOGGLE_COLOR);
+    dialog->b[0] = new Fl_Check_Button(10, y, 180, 25, "Save only modified options"); y += 25;
+    dialog->b[0]->value(1);
+    dialog->b[0]->type(FL_TOGGLE_BUTTON);
+    dialog->b[0]->down_box(GMSH_TOGGLE_BOX);
+    dialog->b[0]->selection_color(GMSH_TOGGLE_COLOR);
+    dialog->b[1] = new Fl_Check_Button(10, y, 180, 25, "Print help strings"); y += 25;
+    dialog->b[1]->value(1);
+    dialog->b[1]->type(FL_TOGGLE_BUTTON);
+    dialog->b[1]->down_box(GMSH_TOGGLE_BOX);
+    dialog->b[1]->selection_color(GMSH_TOGGLE_COLOR);
     dialog->ok = new Fl_Return_Button(10, y+10, 85, 25, "OK");
     dialog->cancel = new Fl_Button(105, y+10, 85, 25, "Cancel");
     dialog->window->set_modal();
@@ -433,7 +489,7 @@ int options_dialog(char *name)
       Fl_Widget* o = Fl::readqueue();
       if (!o) break;
       if (o == dialog->ok) {
-	Print_Options(0, GMSH_FULLRC, dialog->b->value(), name);
+	Print_Options(0, GMSH_FULLRC, dialog->b[0]->value(), dialog->b[1]->value(), name);
 	dialog->window->hide();
 	return 1;
       }
@@ -452,7 +508,7 @@ int msh_dialog(char *name)
 {
   struct _msh_dialog{
     Fl_Window *window;
-    Fl_Check_Button *b;
+    Fl_Check_Button *b[2];
     Fl_Choice *c;
     Fl_Button *ok, *cancel;
   };
@@ -466,17 +522,21 @@ int msh_dialog(char *name)
 
   if(!dialog){
     dialog = new _msh_dialog;
-    int h = 3*10 + 25 + 2*25, y = 0;
+    int h = 3*10 + 25 + 3*25, y = 0;
     // not a "Dialog_Window" since it is modal 
     dialog->window = new Fl_Double_Window(200, h, "MSH Options"); y = 10;
     dialog->window->box(GMSH_WINDOW_BOX);
     dialog->c = new Fl_Choice(10, y, 130, 25, "Format"); y+= 25;
     dialog->c->menu(versionmenu);
     dialog->c->align(FL_ALIGN_RIGHT);
-    dialog->b = new Fl_Check_Button(10, y, 180, 25, "Save all elements"); y += 25;
-    dialog->b->type(FL_TOGGLE_BUTTON);
-    dialog->b->down_box(GMSH_TOGGLE_BOX);
-    dialog->b->selection_color(GMSH_TOGGLE_COLOR);
+    dialog->b[0] = new Fl_Check_Button(10, y, 180, 25, "Save all (ignore physicals)"); y += 25;
+    dialog->b[0]->type(FL_TOGGLE_BUTTON);
+    dialog->b[0]->down_box(GMSH_TOGGLE_BOX);
+    dialog->b[0]->selection_color(GMSH_TOGGLE_COLOR);
+    dialog->b[1] = new Fl_Check_Button(10, y, 180, 25, "Renumber nodes"); y += 25;
+    dialog->b[1]->type(FL_TOGGLE_BUTTON);
+    dialog->b[1]->down_box(GMSH_TOGGLE_BOX);
+    dialog->b[1]->selection_color(GMSH_TOGGLE_COLOR);
     dialog->ok = new Fl_Return_Button(10, y+10, 85, 25, "OK");
     dialog->cancel = new Fl_Button(105, y+10, 85, 25, "Cancel");
     dialog->window->set_modal();
@@ -485,7 +545,8 @@ int msh_dialog(char *name)
   }
   
   dialog->c->value((CTX.mesh.msh_file_version==1.0) ? 0 : 1);
-  dialog->b->value(CTX.mesh.save_all);
+  dialog->b[0]->value(CTX.mesh.save_all);
+  dialog->b[1]->value(CTX.mesh.renumber_nodes_continuous);
   dialog->window->show();
 
   while(dialog->window->shown()){
@@ -495,7 +556,8 @@ int msh_dialog(char *name)
       if (!o) break;
       if (o == dialog->ok) {
 	opt_mesh_msh_file_version(0, GMSH_SET | GMSH_GUI, dialog->c->value() + 1);
-	opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b->value());
+	opt_mesh_save_all(0, GMSH_SET | GMSH_GUI, dialog->b[0]->value());
+	opt_mesh_renumber_nodes_continuous(0, GMSH_SET | GMSH_GUI, dialog->b[1]->value());
 	CreateOutputFile(name, FORMAT_MSH);
 	dialog->window->hide();
 	return 1;
