@@ -1,6 +1,6 @@
-// $Id: Main.cpp,v 1.84 2005/10/24 15:38:13 geuzaine Exp $
+// $Id: Main.cpp,v 1.87.2.1 2006/03/17 21:16:37 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,12 +25,9 @@
 #include <time.h>
 
 #include "GUI.h"
-
 #include "PluginManager.h"
 #include "Gmsh.h"
 #include "GmshUI.h"
-#include "GmshVersion.h"
-
 #include "Geo.h"
 #include "Mesh.h"
 #include "Draw.h"
@@ -43,10 +40,8 @@
 #include "Numeric.h"
 #include "Solvers.h"
 
-char yyname[256] = "";
-int yyerrorstate = 0;
 Context_T CTX;
-Mesh M, *THEM = NULL, *LOCAL = NULL;
+Mesh M, *THEM = NULL;
 GUI *WID = NULL;
 
 int main(int argc, char *argv[])
@@ -170,12 +165,12 @@ int main(int argc, char *argv[])
   // Say welcome!
 
   Msg(STATUS3N, "Ready");
-  Msg(STATUS1N, "Gmsh %s", GMSH_VERSION);
+  Msg(STATUS1N, "Gmsh %s", Get_GmshVersion());
 
   // Log the following for bug reports
 
   Msg(INFO, "-------------------------------------------------------");
-  Msg(INFO, "Gmsh version   : %s", GMSH_VERSION);
+  Msg(INFO, "Gmsh version   : %s", Get_GmshVersion());
   Msg(INFO, gmsh_os);
   Msg(INFO, "%s%s", gmsh_options, Get_BuildOptions());
   Msg(INFO, gmsh_date);
