@@ -1,7 +1,7 @@
 #ifndef _GEO_H_
 #define _GEO_H_
 
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -96,13 +96,13 @@ typedef struct {
 double evaluate_scalarfunction (char *var, double val, char *funct);
 
 void coherence(char *fich);
-void delet(int p1, char *fich, char *what);
+void delet(List_T *list, char *fich, char *what);
 void add_infile(char *text, char *fich);
 void add_trsfline(int N, int *l, char *fich, char *type, char *typearg, char *pts);
 void add_trsfellisurf(int type, int N, int *l, char *fich, char *dir);
 void add_trsfvol(int N, int *l, char *fich, char *vol);
-void add_charlength(int N, int *l, char *fich, char *lc);
-void add_recosurf(int N, int *l, char *fich);
+void add_charlength(List_T *list, char *fich, char *lc);
+void add_recosurf(List_T *list, char *fich);
 void add_param(char *par, char *value, char *fich);
 void add_point(char *fich, char *x, char *y, char *z, char *lc);
 void add_attractor(char *fich, int ip, int typ);
@@ -113,18 +113,22 @@ void add_spline(int N, int *p, char *fich);
 void add_bezier(int N, int *p, char *fich);
 void add_bspline(int N, int *p, char *fich);
 void add_multline(int N, int *p, char *fich);
-void add_loop(List_T *list, char *fich, int *numloop);
+void add_lineloop(List_T *list, char *fich, int *numloop);
 void add_surf(List_T *list, char *fich, int support, int typ);
-void add_vol(List_T *list, char *fich, int *numvol);
-void add_multvol(List_T *list, char *fich);
-void add_physical(List_T *list, char *fich, int type, int *num);
-void translate(int add, int s, char *fich, char *what, char *tx, char *ty, char *tz);
-void rotate(int add, int s, char *fich, char *what, char *ax, char *ay, char *az,
+void add_surfloop(List_T *list, char *fich, int *numvol);
+void add_vol(List_T *list, char *fich);
+int add_physical(List_T *list, char *fich, int type);
+void translate(int add, List_T *list, char *fich, char *what,
+	       char *tx, char *ty, char *tz);
+void rotate(int add, List_T *list, char *fich, char *what, 
+	    char *ax, char *ay, char *az,
 	    char *px, char *py, char *pz, char *angle);
-void dilate(int add, int s, char *fich, char *what, char *dx, char *dy, char *dz, char *df);
-void symmetry(int add, int s, char *fich, char *what, char *sa, char *sb, char *sc, char *sd);
-void extrude(int s, char *fich, char *what, char *tx, char *ty, char *tz);
-void protude(int s, char *fich, char *what, char *ax, char *ay, char *az,
+void dilate(int add, List_T *list, char *fich, char *what,
+	    char *dx, char *dy, char *dz, char *df);
+void symmetry(int add, List_T *list, char *fich, char *what, 
+	      char *sa, char *sb, char *sc, char *sd);
+void extrude(List_T *list, char *fich, char *what, char *tx, char *ty, char *tz);
+void protude(List_T *list, char *fich, char *what, char *ax, char *ay, char *az,
 	     char *px, char *py, char *pz, char *angle);
 
 #endif
