@@ -1,6 +1,6 @@
-// $Id: Draw.cpp,v 1.80 2005/11/19 14:59:41 geuzaine Exp $
+// $Id: Draw.cpp,v 1.95 2006/01/16 17:55:43 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,7 +26,6 @@
 #include "Mesh.h"
 #include "Draw.h"
 #include "Context.h"
-#include "MinMax.h"
 #include "Numeric.h"
 
 extern Context_T CTX;
@@ -75,7 +74,9 @@ void Draw3d(void)
   glDepthFunc(GL_LESS);
   glEnable(GL_DEPTH_TEST);
 
+  InitProjection();
   InitRenderModel();
+  InitPosition();
 
   Draw_Mesh(&M);
 }
@@ -121,38 +122,33 @@ void DrawPlugin(void (*draw)(void))
   CTX.mesh.draw = 1;
 }
 
-// Ortho
-
-void Orthogonalize(int x, int y)
+void ClearOpengl(void)
 {
-  double Va, Wa;
+  glClearColor(CTX.UNPACK_RED(CTX.color.bg) / 255.,
+               CTX.UNPACK_GREEN(CTX.color.bg) / 255.,
+               CTX.UNPACK_BLUE(CTX.color.bg) / 255., 0.);
+  glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
+}
 
-  glMatrixMode(GL_PROJECTION);
-  glLoadIdentity();
+// Init
 
-  if(CTX.render_mode == GMSH_SELECT)
-    gluPickMatrix((GLdouble) x,
-                  (GLdouble) (CTX.viewport[3] - y),
-                  5.0, 5.0, (GLint *) CTX.viewport);
-
-  Va = (GLdouble) (CTX.viewport[3] - CTX.viewport[1]) /
+void InitProjection(int xpick, int ypick, int wpick, int hpick)
+{
+  double Va = 
+    (GLdouble) (CTX.viewport[3] - CTX.viewport[1]) /
     (GLdouble) (CTX.viewport[2] - CTX.viewport[0]);
+  double Wa = (CTX.max[1] - CTX.min[1]) / (CTX.max[0] - CTX.min[0]);
 
-  Wa = (CTX.max[1] - CTX.min[1]) / (CTX.max[0] - CTX.min[0]);
-
+  // compute the viewport in World coordinates (with margins)
   if(Va > Wa) {
     CTX.vxmin = CTX.min[0];
     CTX.vxmax = CTX.max[0];
-    CTX.vymin =
-      0.5 * (CTX.min[1] + CTX.max[1] - Va * (CTX.max[0] - CTX.min[0]));
-    CTX.vymax =
-      0.5 * (CTX.min[1] + CTX.max[1] + Va * (CTX.max[0] - CTX.min[0]));
+    CTX.vymin = 0.5 * (CTX.min[1] + CTX.max[1] - Va * (CTX.max[0] - CTX.min[0]));
+    CTX.vymax = 0.5 * (CTX.min[1] + CTX.max[1] + Va * (CTX.max[0] - CTX.min[0]));
   }
   else {
-    CTX.vxmin =
-      0.5 * (CTX.min[0] + CTX.max[0] - (CTX.max[1] - CTX.min[1]) / Va);
-    CTX.vxmax =
-      0.5 * (CTX.min[0] + CTX.max[0] + (CTX.max[1] - CTX.min[1]) / Va);
+    CTX.vxmin = 0.5 * (CTX.min[0] + CTX.max[0] - (CTX.max[1] - CTX.min[1]) / Va);
+    CTX.vxmax = 0.5 * (CTX.min[0] + CTX.max[0] + (CTX.max[1] - CTX.min[1]) / Va);
     CTX.vymin = CTX.min[1];
     CTX.vymax = CTX.max[1];
   }
@@ -161,39 +157,105 @@ void Orthogonalize(int x, int y)
   CTX.vymin -= (CTX.vymax - CTX.vymin) / 3.;
   CTX.vymax += 0.25 * (CTX.vymax - CTX.vymin);
 
-  CTX.pixel_equiv_x =
-    (CTX.vxmax - CTX.vxmin) / (CTX.viewport[2] - CTX.viewport[0]);
-  CTX.pixel_equiv_y =
-    (CTX.vymax - CTX.vymin) / (CTX.viewport[3] - CTX.viewport[1]);
+  // store what one pixel represents in world coordinates
+  CTX.pixel_equiv_x = (CTX.vxmax - CTX.vxmin) / (CTX.viewport[2] - CTX.viewport[0]);
+  CTX.pixel_equiv_y = (CTX.vymax - CTX.vymin) / (CTX.viewport[3] - CTX.viewport[1]);
+
+  // no initial translation of the model
+  CTX.t_init[0] = CTX.t_init[1] = CTX.t_init[2] = 0.;
+
+  // setup ortho or perspective projection matrix
+  glMatrixMode(GL_PROJECTION);
+  glLoadIdentity();
+
+  // restrict picking to a rectangular region around xpick,ypick in SELECT mode
+  if(CTX.render_mode == GMSH_SELECT)
+    gluPickMatrix((GLdouble)xpick, (GLdouble)(CTX.viewport[3] - ypick),
+                  (GLdouble)wpick, (GLdouble)hpick, (GLint *)CTX.viewport);
+
+  double grad_z, grad_xy;
+  double zmax = MAX(fabs(CTX.min[2]), fabs(CTX.max[2]));
+  if(zmax < CTX.lc) zmax = CTX.lc;
 
-  // We should have a look at how the scaling is done in "real" opengl
-  // applications (I guess they normalize the scene to fit in a 1x1x1
-  // box or something...). Here, we set up a large box around the
-  // object, so that if we zoom a lot the resolution of the depth
-  // buffer might become insufficient (at least with the "software"
-  // Mesa on Linux; with hardware acceleration or on Windows
-  // everyhting seems to be fine).
   if(CTX.ortho) {
-    double maxz = MAX(fabs(CTX.min[2]), fabs(CTX.max[2]));
-    if(maxz < CTX.lc) maxz = CTX.lc;
-    double clip = maxz * CTX.s[2] * CTX.clip_factor;
+    // setting up the near and far clipping planes so that the box is
+    // large enough to manipulate the model and zoom, but not too big
+    // (the z-buffer resolution, e.g., on software Mesa can become
+    // insufficient)
+    double clip = zmax * CTX.s[2] * CTX.clip_factor;
     glOrtho(CTX.vxmin, CTX.vxmax, CTX.vymin, CTX.vymax, -clip, clip);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
+    grad_z = 0.99 * clip;
+    grad_xy = 1.;
   }
   else {
-    glFrustum(CTX.vxmin, CTX.vxmax, CTX.vymin, CTX.vymax, CTX.lc,
-              100 * CTX.lc);
+    double clip_near = 0.75 * CTX.clip_factor * zmax;
+    double clip_far = 75. * CTX.clip_factor * zmax;
+    double coef = (75./0.75) / 3.;
+    // recenter the model such that the perspective is always at the
+    // center of gravity (we should maybe add an option to choose
+    // this, as we do for the rotation center)
+    CTX.t_init[0] = CTX.cg[0];
+    CTX.t_init[1] = CTX.cg[1];
+    CTX.vxmin -= CTX.t_init[0];
+    CTX.vxmax -= CTX.t_init[0];
+    CTX.vymin -= CTX.t_init[1];
+    CTX.vymax -= CTX.t_init[1];
+    glFrustum(CTX.vxmin, CTX.vxmax, CTX.vymin, CTX.vymax, clip_near, clip_far);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
-    glTranslated(0.0, 0.0, -10 * CTX.lc);
-    glScaled(10., 10., 10.);
+    glTranslated(-coef * CTX.t_init[0], -coef * CTX.t_init[1], -coef * clip_near);
+    glScaled(coef, coef, coef);
+    grad_z = 0.99 * clip_far;
+    grad_xy = clip_far / clip_near;
   }
 
+  // draw background gradient
+  if(CTX.render_mode != GMSH_SELECT && CTX.bg_gradient){
+    glPushMatrix();
+    glLoadIdentity();
+    glTranslated(0., 0., -grad_z);
+    if(CTX.bg_gradient == 1){ // vertical
+      glBegin(GL_QUADS);
+      glColor4ubv((GLubyte *) & CTX.color.bg);
+      glVertex3d(grad_xy * CTX.vxmin, grad_xy * CTX.vymin, 0.);
+      glVertex3d(grad_xy * CTX.vxmax, grad_xy * CTX.vymin, 0.);
+      glColor4ubv((GLubyte *) & CTX.color.bg_grad);
+      glVertex3d(grad_xy * CTX.vxmax, grad_xy * CTX.vymax, 0.);
+      glVertex3d(grad_xy * CTX.vxmin, grad_xy * CTX.vymax, 0.);
+      glEnd();
+    }
+    else if(CTX.bg_gradient == 2){ // horizontal
+      glBegin(GL_QUADS);
+      glColor4ubv((GLubyte *) & CTX.color.bg);
+      glVertex3d(grad_xy * CTX.vxmax, grad_xy * CTX.vymin, 0.);
+      glVertex3d(grad_xy * CTX.vxmax, grad_xy * CTX.vymax, 0.);
+      glColor4ubv((GLubyte *) & CTX.color.bg_grad);
+      glVertex3d(grad_xy * CTX.vxmin, grad_xy * CTX.vymax, 0.);
+      glVertex3d(grad_xy * CTX.vxmin, grad_xy * CTX.vymin, 0.);
+      glEnd();
+    }
+    else{ // radial
+      double cx = grad_xy * (CTX.vxmin + CTX.vxmax) / 2.;
+      double cy = grad_xy * (CTX.vymin + CTX.vymax) / 2.;
+      double r = grad_xy * MAX(CTX.vxmax - CTX.vxmin, CTX.vymax - CTX.vymin) / 2.;
+      glBegin(GL_TRIANGLE_FAN);
+      glColor4ubv((GLubyte *) & CTX.color.bg_grad);
+      glVertex3d(cx, cy, 0.);
+      glColor4ubv((GLubyte *) & CTX.color.bg);
+      glVertex3d(cx + r, cy, 0.);
+      int ntheta = 36;
+      for(int i = 1; i < ntheta + 1; i ++){
+	double theta = i * 2 * M_PI / (double)ntheta;
+	glVertex3d(cx + r * cos(theta), cy + r * sin(theta), 0.);	
+      }
+      glEnd();
+    }
+    glPopMatrix();
+  }
 }
 
-// Init
-
 void InitRenderModel(void)
 {
   GLfloat r, g, b;
@@ -206,21 +268,21 @@ void InitRenderModel(void)
 			     (GLfloat)CTX.light_position[i][3]};
       glLightfv((GLenum)(GL_LIGHT0 + i), GL_POSITION, position);
 
-      r = UNPACK_RED(CTX.color.ambient_light[i])/255.;
-      g = UNPACK_GREEN(CTX.color.ambient_light[i])/255.;
-      b = UNPACK_BLUE(CTX.color.ambient_light[i])/255.;
+      r = CTX.UNPACK_RED(CTX.color.ambient_light[i])/255.;
+      g = CTX.UNPACK_GREEN(CTX.color.ambient_light[i])/255.;
+      b = CTX.UNPACK_BLUE(CTX.color.ambient_light[i])/255.;
       GLfloat ambient[4] = {r, g, b, 1.0};
       glLightfv((GLenum)(GL_LIGHT0 + i), GL_AMBIENT, ambient);
 
-      r = UNPACK_RED(CTX.color.diffuse_light[i])/255.;
-      g = UNPACK_GREEN(CTX.color.diffuse_light[i])/255.;
-      b = UNPACK_BLUE(CTX.color.diffuse_light[i])/255.;
+      r = CTX.UNPACK_RED(CTX.color.diffuse_light[i])/255.;
+      g = CTX.UNPACK_GREEN(CTX.color.diffuse_light[i])/255.;
+      b = CTX.UNPACK_BLUE(CTX.color.diffuse_light[i])/255.;
       GLfloat diffuse[4] = {r, g, b, 1.0};
       glLightfv((GLenum)(GL_LIGHT0 + i), GL_DIFFUSE, diffuse);
 
-      r = UNPACK_RED(CTX.color.specular_light[i])/255.;
-      g = UNPACK_GREEN(CTX.color.specular_light[i])/255.;
-      b = UNPACK_BLUE(CTX.color.specular_light[i])/255.;
+      r = CTX.UNPACK_RED(CTX.color.specular_light[i])/255.;
+      g = CTX.UNPACK_GREEN(CTX.color.specular_light[i])/255.;
+      b = CTX.UNPACK_BLUE(CTX.color.specular_light[i])/255.;
       GLfloat specular[4] = {r, g, b, 1.0};
       glLightfv((GLenum)(GL_LIGHT0 + i), GL_SPECULAR, specular);
 
@@ -242,6 +304,7 @@ void InitRenderModel(void)
   glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, CTX.shine_exponent);
 
   glShadeModel(GL_SMOOTH);
+
   // Normalize the normals automatically. We could use the more
   // efficient glEnable(GL_RESCALE_NORMAL) instead (since we initially
   // specify unit normals), but GL_RESCALE_NORMAL does only work with
@@ -285,8 +348,23 @@ void InitPosition(void)
 
 // Entity selection
 
-void Process_SelectionBuffer(int x, int y, int *n, hit *hits)
+typedef struct{
+  unsigned int type, ient, depth;
+} hit;
+
+int fcmp_hit_depth(const void *a, const void *b)
 {
+  return ((hit*)a)->depth - ((hit*)b)->depth;
+}
+
+int Process_SelectionBuffer(int type, bool multi,
+			    int x, int y, int w, int h,
+			    Vertex *v[SELECTION_MAX_HITS],
+			    Curve *c[SELECTION_MAX_HITS],
+			    Surface *s[SELECTION_MAX_HITS],
+			    Mesh *m)
+{
+  hit hits[SELECTION_BUFFER_SIZE];
   GLuint selectBuf[SELECTION_BUFFER_SIZE];
 
   glSelectBuffer(SELECTION_BUFFER_SIZE, selectBuf);
@@ -295,91 +373,93 @@ void Process_SelectionBuffer(int x, int y, int *n, hit *hits)
   CTX.render_mode = GMSH_SELECT;
 
   glInitNames();
-  glPushName(0); // init stack with 0 (=type=point). This will be
-		 // overwritten with the actual entity type everytime
-		 // an entity is drawn
-
   glPushMatrix();
-  Orthogonalize(x, y);
+  InitProjection(x, y, w, h);
+  InitPosition();
   Draw_Mesh(&M);
   glPopMatrix();
 
   GLint numhits = glRenderMode(GL_RENDER);
   CTX.render_mode = GMSH_RENDER;
 
-  if(numhits < 0){
-    // selection buffer overflow
-    *n = 0;
-    return;
+  if(!numhits){ // no hits
+    return 0;
+  }
+  else if(numhits < 0){ // overflow
+    Msg(WARNING, "Selection buffer size exceeded");
+    return 0;
   }
 
   GLint *ptr = (GLint *) selectBuf;
 
+  int numentities = 0;
   for(int i = 0; i < numhits; i++) {
-    GLint names = *ptr; // number of names in the name stack (should
-			// always be 2 in Gmsh: the first is the type
-			// of the entity, the second the entity
-			// number)
-    ptr++;
-    GLint mindepth = *ptr;
-    ptr++;
-    GLint maxdepth = *ptr;
-    ptr++;
-    hits[i].depth = (mindepth+maxdepth)/2;
-    for(int j = 0; j < names; j++) {
-      if(j == 0)
-        hits[i].type = *ptr; // type of entity (0, 1, 2, ... for
-			     // point, line, surf, ...)
-      else if(j == 1)
-        hits[i].ient = *ptr; // num of entity
-      ptr++;
+    GLint names = *ptr++; 
+    GLint mindepth = *ptr++;
+    GLint maxdepth = *ptr++;
+    // in Gmsh 'names' should always be 2 (in which case the first is
+    // the type of the entity (0=point, 1=line, ...) and the second
+    // the entity number) or 0 (if there is nothing in the stack)
+    if(names == 2){
+      hits[numentities].depth = (mindepth+maxdepth)/2;
+      hits[numentities].type = *ptr++; 
+      hits[numentities].ient = *ptr++;
+      numentities++;
     }
   }
-  *n = numhits;
-}
 
-int fcmp_hit_depth(const void *a, const void *b)
-{
-  return ((hit*)a)->depth - ((hit*)b)->depth;
-}
+  if(!numentities){ // no entities
+    return 0;
+  }
 
-int Filter_SelectionBuffer(int type, int n, hit *hits, 
-			   Vertex **thev, Curve **thec, Surface **thes, Mesh *m)
-{
-  // If type == ENT_NONE, return the closest entity of "lowest
-  // dimension" (point < line < surface < volume). Otherwise, return
-  // the closest entity of type "type"
+  // filter result: if type == ENT_NONE, return the closest entity of
+  // "lowest dimension" (point < line < surface < volume). Otherwise,
+  // return the closest entity of type "type"
 
   unsigned int typmin = 4;
-
-  for(int i = 0; i < n; i++) {
+  for(int i = 0; i < numentities; i++) {
     if(hits[i].type < typmin)
       typmin = hits[i].type;
   }
 
   // sort hits to get closest entities first
-  qsort(hits, n, sizeof(hit), fcmp_hit_depth);
-
-  for(int i = 0; i < n; i++) {
+  qsort(hits, numentities, sizeof(hit), fcmp_hit_depth);
+  
+  int j = 0;
+  for(int i = 0; i < numentities; i++) {
     if((type == ENT_NONE && hits[i].type == typmin) ||
        (type == ENT_POINT && hits[i].type == 0) ||
        (type == ENT_LINE && hits[i].type == 1) ||
        (type == ENT_SURFACE && hits[i].type == 2)){
       switch (hits[i].type) {
       case 0:
-	*thev = FindPoint(hits[i].ient, m);
-        return 1;
+	if(!(v[j] = FindPoint(hits[i].ient, m))){
+	  Msg(GERROR, "Problem in point selection processing");
+	  return j;
+	}
+	j++;
+	if(!multi) return 1;
+	break;
       case 1:
-	*thec = FindCurve(hits[i].ient, m);
-	return 1;
+	if(!(c[j] = FindCurve(hits[i].ient, m))){
+	  Msg(GERROR, "Problem in line selection processing");
+	  return j;
+	}
+	j++;
+	if(!multi) return 1;
+	break;
       case 2:
-	*thes = FindSurface(hits[i].ient, m);
-	return 1;
+	if(!(s[j] = FindSurface(hits[i].ient, m))){
+	  Msg(GERROR, "Problem in surface selection processing");
+	  return j;
+	}
+	j++;
+	if(!multi) return 1;
+	break;
       }
     }
   }
-  
-  return 0;
+  return j;
 }
 
 // Takes a cursor position in window coordinates and returns the line
