@@ -1,6 +1,6 @@
-// $Id: Iso.cpp,v 1.34 2005/05/21 17:27:03 geuzaine Exp $
+// $Id: Iso.cpp,v 1.36 2006/01/14 16:24:54 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -390,8 +390,7 @@ void EnhanceSimplexPolygon(Post_View * View, int nb,    // nb of points in polyg
         norms[3 * i + 1] = n[1];
         norms[3 * i + 2] = n[2];
         View->normals->get(Xp[i], Yp[i], Zp[i], 
-			   norms[3 * i], norms[3 * i + 1], norms[3 * i + 2],
-			   View->AngleSmoothNormals);
+			   norms[3 * i], norms[3 * i + 1], norms[3 * i + 2]);
       }
     }
   }
