@@ -1,6 +1,6 @@
-// $Id: Mesh.cpp,v 1.142 2005/11/20 03:58:29 geuzaine Exp $
+// $Id: Mesh.cpp,v 1.151.2.1 2006/04/05 21:32:01 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -53,35 +53,32 @@ static Volume *theVolume = NULL;
 static int numLabelsDisplayed = 0;
 static int stepLabelsDisplayed = 1;
 
-void draw_polygon_2d(double r, double g, double b, int n,
-                     double *x, double *y, double *z)
+void draw_triangle_overlay(double r, double g, double b,
+			   Vertex *v1, Vertex *v2, Vertex *v3)
 {
-  int i;
-
-  CalculateMinMax(THEM->Points, NULL);
-  InitOpengl();
+  // this routine is only used to display the interactive construction
+  // of the 2D aniso mesh
+  SetOpenglContext();
+  InitProjection();
   InitPosition();
-
   glDisable(GL_DEPTH_TEST);
   glDrawBuffer(GL_FRONT);
   glColor3f(r, g, b);
-  glBegin(GL_LINE_STRIP);
-  for(i = 0; i < n; i++)
-    if(z)
-      glVertex3d(x[i], y[i], z[i]);
-    else
-      glVertex2d(x[i], y[i]);
+  glBegin(GL_LINE_LOOP);
+  glVertex3d(v1->Pos.X, v1->Pos.Y, v1->Pos.Z);
+  glVertex3d(v2->Pos.X, v2->Pos.Y, v2->Pos.Z);
+  glVertex3d(v3->Pos.X, v3->Pos.Y, v3->Pos.Z);
   glEnd();
   glFlush();
   glDrawBuffer(GL_BACK);
   glEnable(GL_DEPTH_TEST);
 }
 
-int getFirstPhysical(int type, int num)
+int getLastPhysical(int type, int num)
 {
   // If we visualize the mesh by coloring the physical entities, or if
   // we ask to display physical entity labels on elements, this
-  // routine returns the number of the first physical entity of type
+  // routine returns the number of the last physical entity of type
   // "type" that contains the elementary entity "num"
   if(CTX.mesh.color_carousel == 2 || CTX.mesh.label_type == 2){
     static int warn = 1;
@@ -91,7 +88,7 @@ int getFirstPhysical(int type, int num)
       Msg(WARNING, "You might want to color and/or label the mesh by elementary entity instead");
       warn = 0;
     }
-    for(int i = 0; i < List_Nbr(THEM->PhysicalGroups); i++){
+    for(int i = List_Nbr(THEM->PhysicalGroups)-1; i >=0 ; i--){
       PhysicalGroup *p = *(PhysicalGroup**)List_Pointer(THEM->PhysicalGroups, i);
       if(p->Typ == type && List_Query(p->Entities, &num, fcmp_int))
 	return p->Num;
@@ -138,10 +135,10 @@ int getPartition(int index)
     return (*part)->Num; // partition number
 }
 
+static int preproNormals = 0;
+
 void Draw_Mesh(Mesh * M)
 {
-  InitPosition();
-
   for(int i = 0; i < 6; i++)
     glClipPlane((GLenum)(GL_CLIP_PLANE0 + i), CTX.clip_plane[i]);
 
@@ -266,6 +263,11 @@ void Draw_Mesh(Mesh * M)
       else
 	glDisable((GLenum)(GL_CLIP_PLANE0 + i));
 
+    if(CTX.mesh.changed){
+      if(M->normals) delete M->normals;
+      M->normals = new smooth_normals(CTX.mesh.angle_smooth_normals);
+    }
+
     // Dimension 3
 
     if(M->status >= 3 && (CTX.mesh.volumes_faces || CTX.mesh.volumes_edges ||
@@ -280,6 +282,11 @@ void Draw_Mesh(Mesh * M)
     if(M->status >= 2 && (CTX.mesh.surfaces_faces || CTX.mesh.surfaces_edges ||
 			  CTX.mesh.surfaces_num || CTX.mesh.points_per_element ||
 			  CTX.mesh.normals)) {
+      if(CTX.mesh.changed && CTX.mesh.smooth_normals){
+	preproNormals = 1;
+	Tree_Action(M->Surfaces, Draw_Mesh_Surface);
+	preproNormals = 0;
+      }
       Tree_Action(M->Surfaces, Draw_Mesh_Surface);
       if(CTX.mesh.oldxtrude)  //old extrusion algo
 	Tree_Action(M->Volumes, Draw_Mesh_Extruded_Surfaces);
@@ -341,10 +348,10 @@ void Draw_Mesh_Volume(void *a, void *b)
   // volume cuts drawn "as surfaces" (using vertex arrays for
   // everything would require quite a bit of memory...)
   if(CTX.mesh.use_cut_plane && CTX.mesh.cut_plane_as_surface && 
-     CTX.mesh.vertex_arrays){
+     CTX.mesh.vertex_arrays && (CTX.mesh.surfaces_faces || CTX.mesh.surfaces_edges)){
     if(CTX.mesh.changed){
       Msg(DEBUG, "regenerate volume mesh vertex arrays");
-      thePhysical = getFirstPhysical(MSH_PHYSICAL_VOLUME, v->Num);
+      thePhysical = getLastPhysical(MSH_PHYSICAL_VOLUME, v->Num);
       // triangles
       if(v->TriVertexArray) delete v->TriVertexArray;
       v->TriVertexArray = new VertexArray(3, 1000);
@@ -381,7 +388,7 @@ void Draw_Mesh_Volume(void *a, void *b)
      CTX.mesh.dual || CTX.mesh.volumes_num || CTX.mesh.points_per_element ||
      CTX.mesh.normals){
     Msg(DEBUG, "classic volume data path");
-    thePhysical = getFirstPhysical(MSH_PHYSICAL_VOLUME, v->Num);
+    thePhysical = getLastPhysical(MSH_PHYSICAL_VOLUME, v->Num);
     Tree_Action(v->Simplexes, Draw_Mesh_Tetrahedron);
     Tree_Action(v->SimplexesBase, Draw_Mesh_Tetrahedron);
     Tree_Action(v->Hexahedra, Draw_Mesh_Hexahedron);
@@ -390,8 +397,6 @@ void Draw_Mesh_Volume(void *a, void *b)
   }
 }
 
-static int preproNormals = 0;
-
 void Draw_Mesh_Surface(void *a, void *b)
 {
   Surface *s = *(Surface **) a;
@@ -399,11 +404,11 @@ void Draw_Mesh_Surface(void *a, void *b)
     return;
 
   if(CTX.render_mode == GMSH_SELECT) {
-    glLoadName(2);
+    glPushName(2);
     glPushName(s->Num);
   }
 
-  if (CTX.mesh.surfaces_num) {
+  if(CTX.mesh.surfaces_num) {
     int numLabels = Tree_Nbr(s->Simplexes) + Tree_Nbr(s->SimplexesBase) 
       + Tree_Nbr(s->Quadrangles);
     numLabelsDisplayed = 0;
@@ -415,24 +420,20 @@ void Draw_Mesh_Surface(void *a, void *b)
   theSurface = s;
   theColor = s->Color;
 
-  if(CTX.mesh.changed && CTX.mesh.smooth_normals){
-    Msg(DEBUG, "pre-processing smooth normals");
-    if(s->normals) delete s->normals;
-    s->normals = new smooth_normals;
-    preproNormals = 1;
+  if(preproNormals){
     Tree_Action(s->Simplexes, Draw_Mesh_Triangle);
     Tree_Action(s->SimplexesBase, Draw_Mesh_Triangle);
     Tree_Action(s->Quadrangles, Draw_Mesh_Quadrangle);
-    preproNormals = 0;
+    return;
   }
 
-  if(CTX.mesh.vertex_arrays){
+  if(CTX.mesh.vertex_arrays && (CTX.mesh.surfaces_faces || CTX.mesh.surfaces_edges)){
     if(CTX.mesh.changed || s->ipar[4] == 2 || s->ipar[4] == -2){
       if(s->ipar[4] == 2) s->ipar[4] = 1;
       if(s->ipar[4] == -2) s->ipar[4] = -1;
 
       Msg(DEBUG, "regenerate surface mesh vertex arrays");
-      thePhysical = getFirstPhysical(MSH_PHYSICAL_SURFACE, s->Num);
+      thePhysical = getLastPhysical(MSH_PHYSICAL_SURFACE, s->Num);
       // triangles
       if(s->TriVertexArray) delete s->TriVertexArray;
       s->TriVertexArray = new VertexArray(3, Tree_Nbr(s->Simplexes) + 
@@ -465,7 +466,7 @@ void Draw_Mesh_Surface(void *a, void *b)
   if(!s->TriVertexArray || CTX.mesh.dual || CTX.mesh.surfaces_num ||
      CTX.mesh.points_per_element || CTX.mesh.normals){
     Msg(DEBUG, "classic triangle data path");
-    thePhysical = getFirstPhysical(MSH_PHYSICAL_SURFACE, s->Num);
+    thePhysical = getLastPhysical(MSH_PHYSICAL_SURFACE, s->Num);
     Tree_Action(s->Simplexes, Draw_Mesh_Triangle);
     Tree_Action(s->SimplexesBase, Draw_Mesh_Triangle);
   }
@@ -473,12 +474,13 @@ void Draw_Mesh_Surface(void *a, void *b)
   if(!s->QuadVertexArray || CTX.mesh.dual || CTX.mesh.surfaces_num ||
      CTX.mesh.points_per_element || CTX.mesh.normals){
     Msg(DEBUG, "classic quadrangle data path");
-    thePhysical = getFirstPhysical(MSH_PHYSICAL_SURFACE, s->Num);
+    thePhysical = getLastPhysical(MSH_PHYSICAL_SURFACE, s->Num);
     Tree_Action(s->Quadrangles, Draw_Mesh_Quadrangle);
   }
 
   if(CTX.render_mode == GMSH_SELECT) {
     glPopName();
+    glPopName();
   }
 
   theSurface = NULL;
@@ -503,7 +505,7 @@ void Draw_Mesh_Curve(void *a, void *b)
     return;
 
   if(CTX.render_mode == GMSH_SELECT) {
-    glLoadName(1);
+    glPushName(1);
     glPushName(c->Num);
   }
 
@@ -518,13 +520,13 @@ void Draw_Mesh_Curve(void *a, void *b)
   theCurve = c;
   theColor = c->Color;
 
-  if(CTX.mesh.vertex_arrays){
+  if(CTX.mesh.vertex_arrays && CTX.mesh.lines){
     if(CTX.mesh.changed || c->ipar[3] == 2 || c->ipar[3] == -2){
       if(c->ipar[3] == 2) c->ipar[3] = 1;
       if(c->ipar[3] == -2) c->ipar[3] = -1;
 
       Msg(DEBUG, "regenerate curve mesh vertex array");
-      thePhysical = getFirstPhysical(MSH_PHYSICAL_LINE, c->Num);
+      thePhysical = getLastPhysical(MSH_PHYSICAL_LINE, c->Num);
       if(c->LinVertexArray) delete c->LinVertexArray;
       c->LinVertexArray = new VertexArray(2, Tree_Nbr(c->Simplexes) + 
 					  Tree_Nbr(c->SimplexesBase));
@@ -543,19 +545,21 @@ void Draw_Mesh_Curve(void *a, void *b)
   if(!c->LinVertexArray || CTX.mesh.lines_num ||
      CTX.mesh.points_per_element || CTX.mesh.tangents){
     Msg(DEBUG, "classic line data path");
-    thePhysical = getFirstPhysical(MSH_PHYSICAL_LINE, c->Num);
+    thePhysical = getLastPhysical(MSH_PHYSICAL_LINE, c->Num);
     Tree_Action(c->Simplexes, Draw_Mesh_Line);
     Tree_Action(c->SimplexesBase, Draw_Mesh_Line);
   }
 
   if(CTX.render_mode == GMSH_SELECT) {
     glPopName();
+    glPopName();
   }
 
   theCurve = NULL;
 }
 
-void Draw_Mesh_Point(int num, double x, double y, double z, int degree, int visible)
+void Draw_Mesh_Point(int num, double x, double y, double z, int degree, int visible,
+		     int elementary=-1, int physical=-1, int partition=-1)
 {
   if(!(visible & VIS_MESH))
     return;
@@ -581,7 +585,28 @@ void Draw_Mesh_Point(int num, double x, double y, double z, int degree, int visi
 
   if(CTX.mesh.points_num) {
     char Num[100];
-    sprintf(Num, "%d", num);
+    if(CTX.mesh.label_type == 4)
+      sprintf(Num, "(%g,%g,%g)", x, y, z);
+    else if(CTX.mesh.label_type == 3){
+      if(partition > 0)
+	sprintf(Num, "%d", partition);
+      else
+	strcpy(Num, "NA");
+    }
+    else if(CTX.mesh.label_type == 2){
+      if(physical > 0)
+	sprintf(Num, "%d", physical);
+      else
+	strcpy(Num, "NA");
+    }
+    else if(CTX.mesh.label_type == 1){
+      if(elementary > 0)
+	sprintf(Num, "%d", elementary);
+      else
+	strcpy(Num, "NA");
+    }
+    else
+      sprintf(Num, "%d", num);
     double offset = 0.3 * (CTX.mesh.point_size + CTX.gl_fontsize) * CTX.pixel_equiv_x;
     glRasterPos3d(x + offset / CTX.s[0],
                   y + offset / CTX.s[1],
@@ -636,7 +661,8 @@ void Draw_Mesh_Line(void *a, void *b)
     Z[i] = Zc + CTX.mesh.explode * (s->V[i]->Pos.Z - Zc);
     if(CTX.mesh.points_per_element)
       Draw_Mesh_Point(s->V[i]->Num, X[i], Y[i], Z[i], 
-		      s->V[i]->Degree, s->V[i]->Visible);
+		      s->V[i]->Degree, s->V[i]->Visible,
+		      s->iEnt, thePhysical, iPart);
   }
   
   if(N == 3){
@@ -648,7 +674,8 @@ void Draw_Mesh_Line(void *a, void *b)
     Z[1] = Zc + CTX.mesh.explode * (s->VSUP[0]->Pos.Z - Zc);
     if(CTX.mesh.points_per_element)
       Draw_Mesh_Point(s->VSUP[0]->Num, X[1], Y[1], Z[1], 
-		      s->VSUP[0]->Degree, s->VSUP[0]->Visible);
+		      s->VSUP[0]->Degree, s->VSUP[0]->Visible,
+		      s->iEnt, thePhysical, iPart);
   }
 
   unsigned int col;
@@ -690,7 +717,9 @@ void Draw_Mesh_Line(void *a, void *b)
 
   if(CTX.mesh.lines_num && (numLabelsDisplayed % stepLabelsDisplayed == 0)) {
     glColor4ubv((GLubyte *) & col);
-    if(CTX.mesh.label_type == 3)
+    if(CTX.mesh.label_type == 4)
+      sprintf(Num, "(%g,%g,%g)", Xc, Yc, Zc);
+    else if(CTX.mesh.label_type == 3)
       sprintf(Num, "%d", iPart);
     else if(CTX.mesh.label_type == 2)
       sprintf(Num, "%d", thePhysical);
@@ -728,10 +757,10 @@ void _triFace(double x0, double y0, double z0,
 
   if(CTX.mesh.light || (theSurface && preproNormals)){
     normal3points(x0, y0, z0, x1, y1, z1, x2, y2, z2, n);
-    if(theSurface && preproNormals){
-      theSurface->normals->add(x0, y0, z0, n[0], n[1], n[2]);
-      theSurface->normals->add(x1, y1, z1, n[0], n[1], n[2]);
-      theSurface->normals->add(x2, y2, z2, n[0], n[1], n[2]);
+    if(preproNormals){
+      THEM->normals->add(x0, y0, z0, n[0], n[1], n[2]);
+      THEM->normals->add(x1, y1, z1, n[0], n[1], n[2]);
+      THEM->normals->add(x2, y2, z2, n[0], n[1], n[2]);
       return;
     }
     glNormal3dv(n);
@@ -739,24 +768,21 @@ void _triFace(double x0, double y0, double z0,
 
   if(CTX.mesh.light && theSurface && CTX.mesh.smooth_normals){
     ns[0] = n[0]; ns[1] = n[1]; ns[2] = n[2];
-    theSurface->normals->get(x0, y0, z0, ns[0], ns[1], ns[2], 
-			     CTX.mesh.angle_smooth_normals);
+    THEM->normals->get(x0, y0, z0, ns[0], ns[1], ns[2]);
     glNormal3dv(ns);    
   }
   glVertex3d(x0, y0, z0);
 
   if(CTX.mesh.light && theSurface && CTX.mesh.smooth_normals){
     ns[0] = n[0]; ns[1] = n[1]; ns[2] = n[2];
-    theSurface->normals->get(x1, y1, z1, ns[0], ns[1], ns[2], 
-			     CTX.mesh.angle_smooth_normals);
+    THEM->normals->get(x1, y1, z1, ns[0], ns[1], ns[2]);
     glNormal3dv(ns);    
   }
   glVertex3d(x1, y1, z1);
 
   if(CTX.mesh.light && theSurface && CTX.mesh.smooth_normals){
     ns[0] = n[0]; ns[1] = n[1]; ns[2] = n[2];
-    theSurface->normals->get(x2, y2, z2, ns[0], ns[1], ns[2], 
-			     CTX.mesh.angle_smooth_normals);
+    THEM->normals->get(x2, y2, z2, ns[0], ns[1], ns[2]);
     glNormal3dv(ns);    
   }
   glVertex3d(x2, y2, z2);
@@ -782,11 +808,11 @@ void _quadFace(double *x, double *y, double *z,
     normal3points(x[i0], y[i0], z[i0],
 		  x[i1], y[i1], z[i1],
 		  x[i2], y[i2], z[i2], n);
-    if(theSurface && preproNormals){
-      theSurface->normals->add(x[i0], y[i0], z[i0], n[0], n[1], n[2]);
-      theSurface->normals->add(x[i1], y[i1], z[i1], n[0], n[1], n[2]);
-      theSurface->normals->add(x[i2], y[i2], z[i2], n[0], n[1], n[2]);
-      theSurface->normals->add(x[i3], y[i3], z[i3], n[0], n[1], n[2]);
+    if(preproNormals){
+      THEM->normals->add(x[i0], y[i0], z[i0], n[0], n[1], n[2]);
+      THEM->normals->add(x[i1], y[i1], z[i1], n[0], n[1], n[2]);
+      THEM->normals->add(x[i2], y[i2], z[i2], n[0], n[1], n[2]);
+      THEM->normals->add(x[i3], y[i3], z[i3], n[0], n[1], n[2]);
       return;
     }
     glNormal3dv(n);
@@ -794,32 +820,28 @@ void _quadFace(double *x, double *y, double *z,
 
   if(CTX.mesh.light && theSurface && CTX.mesh.smooth_normals){
     ns[0] = n[0]; ns[1] = n[1]; ns[2] = n[2];
-    theSurface->normals->get(x[i0], y[i0], z[i0], ns[0], ns[1], ns[2], 
-			     CTX.mesh.angle_smooth_normals);
+    THEM->normals->get(x[i0], y[i0], z[i0], ns[0], ns[1], ns[2]);
     glNormal3dv(ns);    
   }
   glVertex3d(x[i0], y[i0], z[i0]);
 
   if(CTX.mesh.light && theSurface && CTX.mesh.smooth_normals){
     ns[0] = n[0]; ns[1] = n[1]; ns[2] = n[2];
-    theSurface->normals->get(x[i1], y[i1], z[i1], ns[0], ns[1], ns[2], 
-			     CTX.mesh.angle_smooth_normals);
+    THEM->normals->get(x[i1], y[i1], z[i1], ns[0], ns[1], ns[2]);
     glNormal3dv(ns);    
   }
   glVertex3d(x[i1], y[i1], z[i1]);
 
   if(CTX.mesh.light && theSurface && CTX.mesh.smooth_normals){
     ns[0] = n[0]; ns[1] = n[1]; ns[2] = n[2];
-    theSurface->normals->get(x[i2], y[i2], z[i2], ns[0], ns[1], ns[2], 
-			     CTX.mesh.angle_smooth_normals);
+    THEM->normals->get(x[i2], y[i2], z[i2], ns[0], ns[1], ns[2]);
     glNormal3dv(ns);    
   }
   glVertex3d(x[i2], y[i2], z[i2]);
 
   if(CTX.mesh.light && theSurface && CTX.mesh.smooth_normals){
     ns[0] = n[0]; ns[1] = n[1]; ns[2] = n[2];
-    theSurface->normals->get(x[i3], y[i3], z[i3], ns[0], ns[1], ns[2], 
-			     CTX.mesh.angle_smooth_normals);
+    THEM->normals->get(x[i3], y[i3], z[i3], ns[0], ns[1], ns[2]);
     glNormal3dv(ns);    
   }
   glVertex3d(x[i3], y[i3], z[i3]);
@@ -957,7 +979,8 @@ void Draw_Mesh_Triangle(void *a, void *b)
     Z[i] = Zc + CTX.mesh.explode * (s->V[i]->Pos.Z - Zc);
     if(CTX.mesh.points_per_element)
       Draw_Mesh_Point(s->V[i]->Num, X[i], Y[i], Z[i], 
-		      s->V[i]->Degree, s->V[i]->Visible);
+		      s->V[i]->Degree, s->V[i]->Visible,
+		      s->iEnt, thePhysical, iPart);
   }
 
   if(s->VSUP){
@@ -972,7 +995,8 @@ void Draw_Mesh_Triangle(void *a, void *b)
       Z2[i] = Zc + CTX.mesh.explode * (s->VSUP[i]->Pos.Z - Zc);
       if(CTX.mesh.points_per_element)
 	Draw_Mesh_Point(s->VSUP[i]->Num, X2[i], Y2[i], Z2[i], 
-			s->VSUP[i]->Degree, s->VSUP[i]->Visible);
+			s->VSUP[i]->Degree, s->VSUP[i]->Visible,
+			s->iEnt, thePhysical, iPart);
     }
   }
 
@@ -983,15 +1007,14 @@ void Draw_Mesh_Triangle(void *a, void *b)
 		   X[2], Y[2], Z[2], n);
     if(preproNormals){
       for(int i = 0; i < 3; i++)
-	theSurface->normals->add(X[i], Y[i], Z[i], n[0], n[1], n[2]);
+	THEM->normals->add(X[i], Y[i], Z[i], n[0], n[1], n[2]);
       return;
     }
     if(theSurface->TriVertexArray->fill){
       for(int i = 0; i < 3; i++){
 	double ns[3] = {n[0], n[1], n[2]};
 	if(CTX.mesh.smooth_normals)
-	  theSurface->normals->get(X[i], Y[i], Z[i], ns[0], ns[1], ns[2], 
-				   CTX.mesh.angle_smooth_normals);
+	  THEM->normals->get(X[i], Y[i], Z[i], ns[0], ns[1], ns[2]);
 	theSurface->TriVertexArray->add(X[i], Y[i], Z[i], ns[0], ns[1], ns[2], col);
       }
       theSurface->TriVertexArray->num++;
@@ -1060,7 +1083,9 @@ void Draw_Mesh_Triangle(void *a, void *b)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    if(CTX.mesh.label_type == 3)
+    if(CTX.mesh.label_type == 4)
+      sprintf(Num, "(%g,%g,%g)", Xc, Yc, Zc);
+    else if(CTX.mesh.label_type == 3)
       sprintf(Num, "%d", iPart);
     else if(CTX.mesh.label_type == 2)
       sprintf(Num, "%d", thePhysical);
@@ -1147,7 +1172,8 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
     Z[i] = Zc + CTX.mesh.explode * (q->V[i]->Pos.Z - Zc);
     if(CTX.mesh.points_per_element)
       Draw_Mesh_Point(q->V[i]->Num, X[i], Y[i], Z[i], 
-		      q->V[i]->Degree, q->V[i]->Visible);
+		      q->V[i]->Degree, q->V[i]->Visible,
+		      q->iEnt, thePhysical, iPart);
   }
 
   if(q->VSUP){
@@ -1162,7 +1188,8 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
       Z2[i] = Zc + CTX.mesh.explode * (q->VSUP[i]->Pos.Z - Zc);
       if(CTX.mesh.points_per_element)
 	Draw_Mesh_Point(q->VSUP[i]->Num, X2[i], Y2[i], Z2[i], 
-			q->VSUP[i]->Degree, q->VSUP[i]->Visible);
+			q->VSUP[i]->Degree, q->VSUP[i]->Visible,
+			q->iEnt, thePhysical, iPart);
     }
   }
 
@@ -1173,15 +1200,14 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
 		    X[2], Y[2], Z[2], n);
     if(preproNormals){
       for(int i = 0; i < 4; i++)
-	theSurface->normals->add(X[i], Y[i], Z[i], n[0], n[1], n[2]);
+	THEM->normals->add(X[i], Y[i], Z[i], n[0], n[1], n[2]);
       return;
     }
     if(theSurface->QuadVertexArray->fill){
       for(int i = 0; i < 4; i++){
 	double ns[3] = {n[0], n[1], n[2]};
 	if(CTX.mesh.smooth_normals)
-	  theSurface->normals->get(X[i], Y[i], Z[i], ns[0], ns[1], ns[2], 
-				   CTX.mesh.angle_smooth_normals);
+	  THEM->normals->get(X[i], Y[i], Z[i], ns[0], ns[1], ns[2]);
 	theSurface->QuadVertexArray->add(X[i], Y[i], Z[i], ns[0], ns[1], ns[2], col);
       }
       theSurface->QuadVertexArray->num++;
@@ -1250,7 +1276,9 @@ void Draw_Mesh_Quadrangle(void *a, void *b)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    if(CTX.mesh.label_type == 3)
+    if(CTX.mesh.label_type == 4)
+      sprintf(Num, "(%g,%g,%g)", Xc, Yc, Zc);
+    else if(CTX.mesh.label_type == 3)
       sprintf(Num, "%d", iPart);
     else if(CTX.mesh.label_type == 2)
       sprintf(Num, "%d", thePhysical);
@@ -1339,7 +1367,8 @@ void Draw_Mesh_Tetrahedron(void *a, void *b)
     Z[i] = Zc + CTX.mesh.explode * (s->V[i]->Pos.Z - Zc);
     if(CTX.mesh.points_per_element)
       Draw_Mesh_Point(s->V[i]->Num, X[i], Y[i], Z[i], 
-		      s->V[i]->Degree, s->V[i]->Visible);
+		      s->V[i]->Degree, s->V[i]->Visible,
+		      s->iEnt, thePhysical, iPart);
   }
 
   if(s->VSUP){
@@ -1354,7 +1383,8 @@ void Draw_Mesh_Tetrahedron(void *a, void *b)
       Z2[i] = Zc + CTX.mesh.explode * (s->VSUP[i]->Pos.Z - Zc);
       if(CTX.mesh.points_per_element)
 	Draw_Mesh_Point(s->VSUP[i]->Num, X2[i], Y2[i], Z2[i], 
-			s->VSUP[i]->Degree, s->VSUP[i]->Visible);
+			s->VSUP[i]->Degree, s->VSUP[i]->Visible,
+			s->iEnt, thePhysical, iPart);
     }
   }
 
@@ -1450,7 +1480,9 @@ void Draw_Mesh_Tetrahedron(void *a, void *b)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    if(CTX.mesh.label_type == 3)
+    if(CTX.mesh.label_type == 4)
+      sprintf(Num, "(%g,%g,%g)", Xc, Yc, Zc);
+    else if(CTX.mesh.label_type == 3)
       sprintf(Num, "%d", iPart);
     else if(CTX.mesh.label_type == 2)
       sprintf(Num, "%d", thePhysical);
@@ -1527,7 +1559,8 @@ void Draw_Mesh_Hexahedron(void *a, void *b)
     Z[i] = Zc + CTX.mesh.explode * (h->V[i]->Pos.Z - Zc);
     if(CTX.mesh.points_per_element)
       Draw_Mesh_Point(h->V[i]->Num, X[i], Y[i], Z[i], 
-		      h->V[i]->Degree, h->V[i]->Visible);
+		      h->V[i]->Degree, h->V[i]->Visible,
+		      h->iEnt, thePhysical, iPart);
   }
 
   if(h->VSUP){
@@ -1542,7 +1575,8 @@ void Draw_Mesh_Hexahedron(void *a, void *b)
       Z2[i] = Zc + CTX.mesh.explode * (h->VSUP[i]->Pos.Z - Zc);
       if(CTX.mesh.points_per_element)
 	Draw_Mesh_Point(h->VSUP[i]->Num, X2[i], Y2[i], Z2[i], 
-			h->VSUP[i]->Degree, h->VSUP[i]->Visible);
+			h->VSUP[i]->Degree, h->VSUP[i]->Visible,
+			h->iEnt, thePhysical, iPart);
     }
   }
 
@@ -1650,7 +1684,9 @@ void Draw_Mesh_Hexahedron(void *a, void *b)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    if(CTX.mesh.label_type == 3)
+    if(CTX.mesh.label_type == 4)
+      sprintf(Num, "(%g,%g,%g)", Xc, Yc, Zc);
+    else if(CTX.mesh.label_type == 3)
       sprintf(Num, "%d", iPart);
     else if(CTX.mesh.label_type == 2)
       sprintf(Num, "%d", thePhysical);
@@ -1727,7 +1763,8 @@ void Draw_Mesh_Prism(void *a, void *b)
     Z[i] = Zc + CTX.mesh.explode * (p->V[i]->Pos.Z - Zc);
     if(CTX.mesh.points_per_element)
       Draw_Mesh_Point(p->V[i]->Num, X[i], Y[i], Z[i], 
-		      p->V[i]->Degree, p->V[i]->Visible);
+		      p->V[i]->Degree, p->V[i]->Visible,
+		      p->iEnt, thePhysical, iPart);
   }
 
   if(p->VSUP){
@@ -1747,7 +1784,8 @@ void Draw_Mesh_Prism(void *a, void *b)
       Z2[i] = Zc + CTX.mesh.explode * (p->VSUP[i]->Pos.Z - Zc);
       if(CTX.mesh.points_per_element)
 	Draw_Mesh_Point(p->VSUP[i]->Num, X2[i], Y2[i], Z2[i], 
-			p->VSUP[i]->Degree, p->VSUP[i]->Visible);
+			p->VSUP[i]->Degree, p->VSUP[i]->Visible,
+			p->iEnt, thePhysical, iPart);
     }
   }
 
@@ -1865,7 +1903,9 @@ void Draw_Mesh_Prism(void *a, void *b)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    if(CTX.mesh.label_type == 3)
+    if(CTX.mesh.label_type == 4)
+      sprintf(Num, "(%g,%g,%g)", Xc, Yc, Zc);
+    else if(CTX.mesh.label_type == 3)
       sprintf(Num, "%d", iPart);
     else if(CTX.mesh.label_type == 2)
       sprintf(Num, "%d", thePhysical);
@@ -1942,7 +1982,8 @@ void Draw_Mesh_Pyramid(void *a, void *b)
     Z[i] = Zc + CTX.mesh.explode * (p->V[i]->Pos.Z - Zc);
     if(CTX.mesh.points_per_element)
       Draw_Mesh_Point(p->V[i]->Num, X[i], Y[i], Z[i], 
-		      p->V[i]->Degree, p->V[i]->Visible);
+		      p->V[i]->Degree, p->V[i]->Visible,
+		      p->iEnt, thePhysical, iPart);
   }
 
   if(p->VSUP){
@@ -1962,7 +2003,8 @@ void Draw_Mesh_Pyramid(void *a, void *b)
       Z2[i] = Zc + CTX.mesh.explode * (p->VSUP[i]->Pos.Z - Zc);
       if(CTX.mesh.points_per_element)
 	Draw_Mesh_Point(p->VSUP[i]->Num, X2[i], Y2[i], Z2[i], 
-			p->VSUP[i]->Degree, p->VSUP[i]->Visible);
+			p->VSUP[i]->Degree, p->VSUP[i]->Visible,
+			p->iEnt, thePhysical, iPart);
     }
   }
 
@@ -2052,7 +2094,9 @@ void Draw_Mesh_Pyramid(void *a, void *b)
       glColor4ubv((GLubyte *) & CTX.color.mesh.line);
     else
       glColor4ubv((GLubyte *) & col);
-    if(CTX.mesh.label_type == 3)
+    if(CTX.mesh.label_type == 4)
+      sprintf(Num, "(%g,%g,%g)", Xc, Yc, Zc);
+    else if(CTX.mesh.label_type == 3)
       sprintf(Num, "%d", iPart);
     else if(CTX.mesh.label_type == 2)
       sprintf(Num, "%d", thePhysical);
