@@ -1,6 +1,6 @@
-// $Id: Post.cpp,v 1.102 2005/09/02 18:04:02 geuzaine Exp $
+// $Id: Post.cpp,v 1.105 2006/02/14 15:13:18 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -439,12 +439,24 @@ void Draw_List(Post_View * v, double ValMin, double ValMax, int type,
 
 int EstimateNumTri(Post_View *v)
 {
-  int num2d = v->NbST + v->NbSQ;
-  int num3d = v->NbSS + v->NbSH + v->NbSI + v->NbSY;
+  int tris = v->NbST + v->NbVT + v->NbTT;
+  int quads = v->NbSQ + v->NbVQ + v->NbTQ;
+  int tets = v->NbSS + v->NbVS + v->NbTS;
+  int prisms = v->NbSI + v->NbVI + v->NbTI;
+  int pyrs = v->NbSY + v->NbVY + v->NbTY;
+  int hexas = v->NbSH + v->NbVH + v->NbTH;
+  
+  int heuristic = 0;
+  if(v->IntervalsType == DRAW_POST_ISO)
+    heuristic = (tets + prisms + pyrs + hexas) / 10;
+  else if(v->IntervalsType == DRAW_POST_CONTINUOUS)
+    heuristic = (tris + 2 * quads + 6 * tets + 
+      8 * prisms + 6 * pyrs + 12 * hexas);
+  else if(v->IntervalsType == DRAW_POST_DISCRETE)
+    heuristic = (tris + 2 * quads + 6 * tets + 
+      8 * prisms + 6 * pyrs + 12 * hexas) * 2;
 
-  int heuristic = num2d + num3d/10;
-
-  return (heuristic < 10000) ? 10000 : heuristic;
+  return heuristic + 10000;
 }
 
 int EstimateNumLin(Post_View *v)
