@@ -1,6 +1,6 @@
-// $Id: 1D_Mesh.cpp,v 1.45 2005/05/15 01:44:26 geuzaine Exp $
+// $Id: 1D_Mesh.cpp,v 1.48 2006/01/29 22:53:41 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -59,7 +59,8 @@ double F_Transfini(double t)
   d = sqrt(der.Pos.X * der.Pos.X + der.Pos.Y * der.Pos.Y +
            der.Pos.Z * der.Pos.Z);
 
-  if(THEC->dpar[0] == 0.0 || THEC->dpar[0] == 1.0) {
+  if(THEC->dpar[0] <= 0.0 || THEC->dpar[0] == 1.0) {
+    // dpar[0] < 0 should never happen
     val = d * (double)THEC->ipar[0] / THEC->l;
   }
   else {
@@ -70,10 +71,7 @@ double F_Transfini(double t)
 	r = THEC->dpar[0];
       else
 	r = 1. / THEC->dpar[0];
-      if(r == 1.)
-        a = THEC->l / (double)THEC->ipar[0];
-      else
-        a = THEC->l * (r - 1.) / (pow(r, THEC->ipar[0]) - 1.);
+      a = THEC->l * (r - 1.) / (pow(r, THEC->ipar[0] - 1.) - 1.);
       i = (int)(log(t * THEC->l / a * (r - 1.) + 1.) / log(r));
       val = d / (a * pow(r, (double)i));
       break;
@@ -108,10 +106,10 @@ double F_Lc(double t)
   Vertex der, point;
   double Lc, d;
 
-  if(THEM->BGM.Typ == ONFILE) {
+  if(THEM->BackgroundMeshType == ONFILE) {
     der = InterpolateCurve(THEC, t, 1);
     point = InterpolateCurve(THEC, t, 0);
-    Lc = Lc_XYZ(point.Pos.X, point.Pos.Y, point.Pos.Z, THEM);
+    Lc = BGMXYZ(point.Pos.X, point.Pos.Y, point.Pos.Z);
     d = sqrt(DSQR(der.Pos.X) + DSQR(der.Pos.Y) + DSQR(der.Pos.Z));
     if(CTX.mesh.constrained_bgmesh)
       return MAX(d / Lc, THEM->Metric->getLc(t, THEC));
