@@ -1,7 +1,7 @@
 #ifndef _2D_MESH_H_
 #define _2D_MESH_H_
 
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ Delaunay * Find_Triangle (MPoint pt, DocRecord *MESH, int typ);
 int Insert_Triangle (avlstruct **root, Delaunay * del);
 int Delete_Triangle ( avlstruct **root, Delaunay * del );
 int Insert_Point (MPoint pt, int *numpoints, int *numalloc, 
-                  DocRecord *doc, DocRecord *BGM, int is3d);
+                  DocRecord *doc, DocRecord *BGM);
 MPoint Localize (Delaunay * del , DocRecord *MESH);
 
 void InitBricks (DocRecord *MESH);
