@@ -1,6 +1,6 @@
-// $Id: 2D_Mesh_Aniso.cpp,v 1.45 2005/06/20 17:02:45 geuzaine Exp $
+// $Id: 2D_Mesh_Aniso.cpp,v 1.48.2.1 2006/04/05 21:32:01 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -61,12 +61,12 @@ double Interpole_lcTriangle(Simplex * s, Vertex * vv)
 {
   double Xp, Yp, X[3], Y[3], det, u, v, q1, q2, q3;
 
-  if(THEM->BGM.Typ == ONFILE){
+  if(THEM->BackgroundMeshType == ONFILE){
     Vertex *v2 = Create_Vertex(-1, vv->Pos.X, vv->Pos.Y, 0.0, 0.0, 0.0);
     Vertex *dum;
     Calcule_Z_Plan(&v2, &dum);
     Projette_Inverse(&v2, &dum);
-    double val = Lc_XYZ(v2->Pos.X, v2->Pos.Y, v2->Pos.Z, THEM);
+    double val = BGMXYZ(v2->Pos.X, v2->Pos.Y, v2->Pos.Z);
     Free_Vertex(&v2, 0);
     return val;
   }
@@ -215,16 +215,6 @@ void Box_2_Triangles(List_T * P, Surface * s)
   Xc = XM - Xm;
   Yc = YM - Ym;
 
-  /* initialisation de la grille */
-
-  s->Grid.init = 0;
-  s->Grid.min.X = Xm - LOIN * FACT * Xc;
-  s->Grid.min.Y = Ym - LOIN * FACT * Yc;
-  s->Grid.max.X = XM + LOIN * FACT * Xc;
-  s->Grid.max.Y = YM + LOIN * FACT * Yc;
-
-  s->Grid.Nx = s->Grid.Ny = 20;
-
   /* Longueur Caracteristique */
 
   LC2D = sqrt(Xc * Xc + Yc * Yc);
@@ -537,35 +527,30 @@ int recur_bowyer_2D(Simplex * s)
 
 bool draw_simplex2d(Surface * sur, Simplex * s, bool nouv)
 {
-  Vertex v1, v2, v3;
-
   if(!CTX.mesh.interactive)
     return false;
 
   if(s == &MyNewBoundary || !s || !s->iEnt)
     return false;
 
-  v1 = InterpolateSurface(sur->Support, s->V[0]->Pos.X, s->V[0]->Pos.Y, 0, 0);
-  v2 = InterpolateSurface(sur->Support, s->V[1]->Pos.X, s->V[1]->Pos.Y, 0, 0);
-  v3 = InterpolateSurface(sur->Support, s->V[2]->Pos.X, s->V[2]->Pos.Y, 0, 0);
-
 #if defined(HAVE_FLTK)
-  double x[3], y[3], z[3];
-  x[0] = v1.Pos.X;
-  x[1] = v2.Pos.X;
-  x[2] = v3.Pos.X;
-  y[0] = v1.Pos.Y;
-  y[1] = v2.Pos.Y;
-  y[2] = v3.Pos.Y;
-  z[0] = v1.Pos.Z;
-  z[1] = v2.Pos.Z;
-  z[2] = v3.Pos.Z;
-  void draw_polygon_2d(double r, double g, double b, int n,
-                       double *x, double *y, double *z);
+  Vertex v1 = InterpolateSurface(sur->Support, s->V[0]->Pos.X, s->V[0]->Pos.Y, 0, 0);
+  Vertex v2 = InterpolateSurface(sur->Support, s->V[1]->Pos.X, s->V[1]->Pos.Y, 0, 0);
+  Vertex v3 = InterpolateSurface(sur->Support, s->V[2]->Pos.X, s->V[2]->Pos.Y, 0, 0);
+
+  Vertex *pv1 = &v1, *pv2 = &v2, *pv3 = &v3, *dum;
+  Calcule_Z_Plan(&pv1, &dum);
+  Projette_Inverse(&pv1, &dum);
+  Calcule_Z_Plan(&pv2, &dum);
+  Projette_Inverse(&pv2, &dum);
+  Calcule_Z_Plan(&pv3, &dum);
+  Projette_Inverse(&pv3, &dum);
+
+  void draw_triangle_overlay(double, double, double, Vertex *, Vertex *, Vertex *);
   if(nouv)
-    draw_polygon_2d(1., 0., 0., 3, x, y, z);
+    draw_triangle_overlay(1., 0., 0., pv1, pv2, pv3);
   else
-    draw_polygon_2d(0., 0., 0., 3, x, y, z);
+    draw_triangle_overlay(0., 0., 0., pv1, pv2, pv3);
 #endif
 
   return true;
@@ -972,8 +957,6 @@ Vertex *NewVertex_2D(Simplex * s)
   return (v);
 }
 
-extern Mesh *LOCAL;
-
 void TRIE_MON_GARS(void *a, void *b)
 {
   Simplex *s = *(Simplex **) a;
@@ -1043,7 +1026,6 @@ int AlgorithmeMaillage2DAnisotropeModeJF(Surface * s)
   FACE_DIMENSION = 1;
 
   SURF = s;
-  LOCAL = NULL;
 
   if(s->Typ == MSH_SURF_PLAN || s->Typ == MSH_SURF_REGL
      || s->Typ == MSH_SURF_TRIC)
@@ -1111,6 +1093,21 @@ int AlgorithmeMaillage2DAnisotropeModeJF(Surface * s)
     Tree_Suppress(s->Simplexes, List_Pointer(Suppress, i));
   }
 
+  if(CTX.mesh.initial_only == 2){
+    List = Tree2List(s->Vertices);
+    for(i = 0; i < List_Nbr(List); i++) {
+      List_Read(List, i, &THEV);
+      if(THEV->Num < 0)
+	Tree_Suppress(s->Vertices, &THEV);
+    }
+    List_Delete(List);
+    List_Delete(Points);
+    for(i = 0; i < List_Nbr(Suppress); i++)
+      Free_Simplex(List_Pointer(Suppress, i), 0);
+    List_Delete(Suppress);
+    return 1;
+  }
+
   Msg(STATUS2, "Mesh 2D... (final)");
 
   if(!Tree_Right(s->Simplexes, &simp))
