@@ -1,6 +1,6 @@
-// $Id: 3D_Mesh.cpp,v 1.67 2005/07/03 08:02:24 geuzaine Exp $
+// $Id: 3D_Mesh.cpp,v 1.73 2006/01/31 00:22:33 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 // while the tree of bad quality tets is not empty {
 //   take the worst tet
 //   create a new point
-//   rempve the test whose circumscribed sphere contains the point
+//   remove the test whose circumscribed sphere contains the point
 //   reconstruct the convex volume
 // } 
 
@@ -38,13 +38,13 @@
 #include "Create.h"
 #include "Context.h"
 
-extern Mesh *THEM, *LOCAL;
+extern Mesh *THEM;
 extern Context_T CTX;
 extern int FACE_DIMENSION;
 
 static Tree_T *Tsd, *Sim_Sur_Le_Bord, *POINTS_TREE;
 static List_T *Simplexes_Destroyed, *Simplexes_New, *Suppress;
-static List_T *LLL, *POINTS_LIST;
+static List_T *POINTS_LIST;
 static Simplex *THES;
 static Vertex *THEV;
 static Tree_T *SimXFac;
@@ -160,42 +160,6 @@ Vertex *NewVertex(Simplex * s)
   return (v);
 }
 
-int Pt_In_Volume(double X, double Y, double Z, Mesh * m,
-                 double *l, double tol)
-{
-  int i;
-  Vertex V;
-  double uvw[3];
-  Simplex *s;
-  Brick B;
-
-  V.Pos.X = X;
-  V.Pos.Y = Y;
-  V.Pos.Z = Z;
-
-  if(!(m->BGM.Typ == ONFILE) && !m->BGM.bgm) {
-    *l = -1.0;
-    return (1);
-  }
-
-  B = LaBrique(&m->Grid, X, Y, Z);
-
-  if(B.N < 0) {
-    return (0);
-  }
-
-  for(i = 0; i < List_Nbr(B.pT); i++) {
-    List_Read(B.pT, i, &s);
-    if(s->Pt_In_Simplexe(&V, uvw, tol)) {
-      *l = (1. - uvw[0] - uvw[1] - uvw[2]) * s->V[0]->lc
-        + uvw[0] * s->V[1]->lc + uvw[1] * s->V[2]->lc + uvw[2] * s->V[3]->lc;
-      return (1);
-    }
-  }
-
-  return (0);
-}
-
 inline int Pt_In_Circum(Simplex * s, Vertex * v)
 {
   // Determines of a point is inside the simplexe's circumscribed sphere
@@ -406,16 +370,6 @@ void Box_6_Tetraedron(List_T * P, Mesh * m)
   Yc = YM - Ym;
   Zc = ZM - Zm;
 
-  m->Grid.init = 0;
-  m->Grid.min.X = Xm - LOIN * FACT * Xc;
-  m->Grid.min.Y = Ym - LOIN * FACT * Yc;
-  m->Grid.min.Z = Zm - LOIN * FACT * Zc;
-  m->Grid.max.X = XM + LOIN * FACT * Xc;
-  m->Grid.max.Y = YM + LOIN * FACT * Yc;
-  m->Grid.max.Z = ZM + LOIN * FACT * Zc;
-
-  m->Grid.Nx = m->Grid.Ny = m->Grid.Nz = 20;
-
   LC3D = sqrt(Xc * Xc + Yc * Yc + Zc * Zc);
 
   /* 8 box points
@@ -813,44 +767,6 @@ void suppress_simplex(void *data, void *dum)
     List_Add(Suppress, pv);
 }
 
-void add_in_bgm(void *a, void *b)
-{
-  Simplex **s, *S;
-
-  s = (Simplex **) a;
-  S = *s;
-  List_Add(LLL, S);
-}
-
-void Bgm_With_Points(Mesh * bgm)
-{
-  int i;
-  Simplex *s;
-
-  bgm->BGM.bgm = List_Create(Tree_Nbr(bgm->Simplexes), 10, sizeof(Simplex));
-  LLL = bgm->BGM.bgm;
-  Tree_Action(bgm->Simplexes, add_in_bgm);
-  for(i = 0; i < List_Nbr(LLL); i++) {
-    s = (Simplex *) List_Pointer(LLL, i);
-    AddSimplexInGrid(bgm, s, BOITE);
-  }
-}
-
-void Create_BgMesh(int Type, double lc, Mesh * m)
-{
-  m->BGM.Typ = Type;
-  switch (Type) {
-  case CONSTANT:
-    m->BGM.lc = lc;
-    break;
-  case ONFILE:
-    break;
-  case WITHPOINTS:
-    m->BGM.bgm = NULL;
-    break;
-  }
-}
-
 void Maillage_Volume(void *data, void *dum)
 {
   Volume *v, **pv;
@@ -867,9 +783,16 @@ void Maillage_Volume(void *data, void *dum)
   pv = (Volume **) data;
   v = *pv;
 
-  if(v->Typ == MSH_VOLUME_DISCRETE) {
-  }
-  else if(Extrude_Mesh(v)) {
+  /*  if(v->Typ == MSH_VOLUME_DISCRETE) 
+    {
+      printf("coucou1\n");
+      int temp = CTX.mesh.algo3d;
+      CTX.mesh.algo3d = FRONTAL_NETGEN;
+      CTX.mesh.algo3d =DELAUNAY_TETGEN;
+      Mesh_Tetgen(v);
+      CTX.mesh.algo3d = temp;
+    }
+    else*/ if(Extrude_Mesh(v)) {
   }
   else if(MeshTransfiniteVolume(v)) {
   }
@@ -883,8 +806,7 @@ void Maillage_Volume(void *data, void *dum)
     Simplexes_New = List_Create(10, 10, sizeof(Simplex *));
     Simplexes_Destroyed = List_Create(10, 10, sizeof(Simplex *));
 
-    LOCAL = &M;
-    Create_BgMesh(THEM->BGM.Typ, .2, LOCAL);
+    Mesh *LOCAL = &M;
     s = &S;
 
     POINTS_TREE = Tree_Create(sizeof(Vertex *), comparePosition);
@@ -955,7 +877,6 @@ void Maillage_Volume(void *data, void *dum)
 
     v->Simplexes = LOCAL->Simplexes;
 
-    Bgm_With_Points(LOCAL);
     POINTS_TREE = THEM->Simplexes;
 
     Tree_Right(LOCAL->Simplexes, &simp);
@@ -963,9 +884,6 @@ void Maillage_Volume(void *data, void *dum)
 
     while(simp->Quality > CONV_VALUE) {
       newv = NewVertex(simp);
-      //double l;
-      //while(!Pt_In_Volume(newv->Pos.X,newv->Pos.Y,newv->Pos.Z,LOCAL,&l,0.0)){
-
       while(!simp->Pt_In_Simplexe(newv, uvw, 1.e-5) &&
             (simp->S[0] == &MyNewBoundary ||
              !simp->S[0]->Pt_In_Simplexe(newv, uvw, 1.e-5)) &&
