@@ -1,6 +1,6 @@
-// $Id: BDS.cpp,v 1.44 2005/11/24 19:59:01 geuzaine Exp $
+// $Id: BDS.cpp,v 1.49.2.3 2006/03/18 04:48:29 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -26,6 +26,19 @@
 #include "BDS.h"
 #include "Message.h"
 
+double BDS_Point::min_edge_length()
+{
+  std::list<BDS_Edge*>::iterator it  = edges.begin();
+  std::list<BDS_Edge*>::iterator ite = edges.end();
+  double L = 1.e245;
+  while(it!=ite){
+    double l = (*it)->length();
+    if (l<L)L=l;
+    ++it;
+  }
+  return L;
+}
+
 void outputScalarField(std::list < BDS_Triangle * >t, const char *iii)
 {
   FILE *f = fopen(iii, "w");
@@ -120,7 +133,7 @@ void BDS_Quadric::projection(double xa, double ya, double za,
 
 void BDS_GeomEntity::getClosestTriangles(double x, double y, double z,
                                          std::list < BDS_Triangle * >&l,
-                                         double &radius,
+                                         double &radius ,
                                          double &X, double &Y, double &Z)
 {
 #ifdef HAVE_ANN_
@@ -823,6 +836,7 @@ void BDS_Mesh::createSearchStructures()
           int N = (int)(l / (LC_SEARCH) + 2);
           BDS_Point *p1 = (*eit)->p1;
           BDS_Point *p2 = (*eit)->p2;
+
           for(int i = 0; i < N; i++) {
             double u = (double)i / (N - 1);
             (*it)->dataPts[I][0] = p1->X + (p2->X - p1->X) * (u);
@@ -962,6 +976,9 @@ void recur_color_plane_surf(const double eps,
   }
 }
 
+
+
+
 void BDS_Mesh::color_plane_surf(double eps, int NB_T)
 {
   int current_status = 100000;
@@ -997,6 +1014,22 @@ void BDS_Mesh::color_plane_surf(double eps, int NB_T)
   }
 }
 
+bool BDS_Mesh::extractVolumes()
+{
+  bool closed = true;
+  {
+    std::list < BDS_Edge * >::iterator it = edges.begin();
+    std::list < BDS_Edge * >::iterator ite = edges.end();
+    while(it != ite) 
+      {
+	if ( (*it)->numfaces() !=2 ) closed = false;	  
+	++it;
+      }    
+  }
+  return closed;
+}
+
+
 void BDS_Mesh::classify(double angle, int NB_T)
 {
   // printf("  classifying \n");
@@ -1113,7 +1146,7 @@ void BDS_Mesh::classify(double angle, int NB_T)
       if(e.g) {
         int MIN_FAC = 100000;
         int MAX_FAC = -100000;
-        std::map < std::pair < int, int >, int >::iterator found = 0;
+        std::map < std::pair < int, int >, int >::iterator found;
         BDS_GeomEntity *g;
         if(e.numfaces() == 1) {
           found =
@@ -1246,7 +1279,8 @@ void BDS_Mesh::classify(double angle, int NB_T)
   Msg(INFO, "Creating search structures");
   createSearchStructures();
   Msg(INFO, "End classifying %d edgetags %d vertextags", edgetag - 1, vertextag - 1);
-  // outputScalarField (triangles,"R_curvature.pos");
+  //  outputScalarField (triangles,"R_curvature.pos");
+  extractVolumes();
 }
 
 double PointLessThanLexicographic::t = 0;
@@ -1254,7 +1288,9 @@ double BDS_Vector::t = 0;
 
 bool BDS_Mesh::read_stl(const char *filename, const double tolerance)
 {
-  FILE *f = fopen(filename, "r");
+  // add 'b' for pure Windows programs, since some of these files
+  // contain binary data
+  FILE *f = fopen(filename, "rb");
   if(!f)
     return false;
   char buffer[256], name[256];
@@ -1560,6 +1596,63 @@ bool BDS_Mesh::read_stl(const char *filename, const double tolerance)
   return true;
 }
 
+bool BDS_Mesh::import_view(Post_View *view, const double tolerance)
+{
+  // imports all the tris+quads from a post-processing view
+
+  Min[0] = view->BBox[0]; Max[0] = view->BBox[1];
+  Min[1] = view->BBox[2]; Max[1] = view->BBox[3];
+  Min[2] = view->BBox[4]; Max[2] = view->BBox[5];
+  LC = sqrt((Min[0] - Max[0]) * (Min[0] - Max[0]) +
+	    (Min[1] - Max[1]) * (Min[1] - Max[1]) +
+	    (Min[2] - Max[2]) * (Min[2] - Max[2]));
+  
+  PointLessThanLexicographic::t = tolerance;
+  std::set < BDS_Point *, PointLessThanLexicographic > pts;
+
+  for(int type = 0; type < 6; type++){
+    int nbList, nbNod;
+    List_T *list;
+    switch(type){
+    case 0: list = view->ST; nbList = view->NbST; nbNod = 3; break;
+    case 1: list = view->VT; nbList = view->NbVT; nbNod = 3; break;
+    case 2: list = view->TT; nbList = view->NbTT; nbNod = 3; break;
+    case 3: list = view->SQ; nbList = view->NbSQ; nbNod = 4; break;
+    case 4: list = view->VQ; nbList = view->NbVQ; nbNod = 4; break;
+    case 5: list = view->TQ; nbList = view->NbTQ; nbNod = 4; break;
+    }
+    if(nbList){
+      int nb = List_Nbr(list) / nbList;
+      for(int i = 0; i < List_Nbr(list); i += nb) {
+	double *x = (double *)List_Pointer_Fast(list, i);
+	double *y = (double *)List_Pointer_Fast(list, i + nbNod);
+	double *z = (double *)List_Pointer_Fast(list, i + 2 * nbNod);
+	BDS_Point *p[4];
+	for(int j = 0; j < nbNod; j++){
+	  BDS_Point P(0, x[j], y[j], z[j]);    
+	  std::set < BDS_Point *, PointLessThanLexicographic >::iterator it = pts.find(&P);
+	  if(it != pts.end()) {
+	    p[j] = *it;
+	  }
+	  else {
+	    MAXPOINTNUMBER++;
+	    p[j] = add_point(MAXPOINTNUMBER, P.X, P.Y, P.Z);
+	    pts.insert(p[j]);
+	  }
+	}
+	if(nbNod == 3){
+	  add_triangle(p[0]->iD, p[1]->iD, p[2]->iD);
+	}
+	else{
+	  add_triangle(p[0]->iD, p[1]->iD, p[2]->iD);
+	  add_triangle(p[0]->iD, p[2]->iD, p[3]->iD);
+	}
+      }
+    }
+  }
+  return true;
+}
+
 // INRIA FORMAT
 bool BDS_Mesh::read_mesh(const char *filename)
 {
@@ -1678,7 +1771,7 @@ void BDS_Mesh::save_gmsh_format(const char *filename)
       }
     }
 
-    fprintf(f, "%ld\n", nbClasEdges + nbModelVertex + triangles.size());
+    fprintf(f, "%d\n", nbClasEdges + nbModelVertex + triangles.size());
 
     int k = 1;
     {
@@ -1899,7 +1992,7 @@ bool BDS_Mesh::split_edge(BDS_Edge * e, double coord, BDS_Mesh * geom)
   triangles.push_back(t4);
 
   if(geom || mid->g->surf)
-    snap_point(mid, geom);
+      snap_point(mid, geom);
 
   return true;
 }
@@ -1953,6 +2046,32 @@ bool BDS_Mesh::swap_edge(BDS_Edge * e)
     }
   }
 
+  // See if the swap reverts one triangle
+  double cb1[3],cb2[3];
+  if(orientation == 1) 
+    {    
+      normal_triangle ( p1 , op[1] , op[0] , cb1);
+      normal_triangle ( op[1] , p2 , op[0] , cb2);  
+    }
+  else
+    {    
+      normal_triangle ( p1 , op[0] , op[1]  , cb1);
+      normal_triangle ( op[0] , p2 , op[1] , cb2);  
+    }
+
+  BDS_Vector n1 = e->faces(0)->N();
+  BDS_Vector n2 = e->faces(1)->N();
+  double psc = n1.x * cb1[0] +n1.y * cb1[1] +n1.z * cb1[2];
+  if (psc < 0) return false;
+  psc = n2.x * cb1[0] +n2.y * cb1[1] +n2.z * cb1[2];
+  if (psc < 0) return false;
+  psc = n2.x * cb2[0] +n2.y * cb2[1] +n2.z * cb2[2];
+  if (psc < 0) return false;
+  psc = n1.x * cb2[0] +n1.y * cb2[1] +n1.z * cb2[2];
+  if (psc < 0) return false;
+  //-----------------------------------------------
+
+
   BDS_Edge *p1_op1 = find_edge(p1, op[0], e->faces(0));
   BDS_Edge *op1_p2 = find_edge(op[0], p2, e->faces(0));
   BDS_Edge *p1_op2 = find_edge(p1, op[1], e->faces(1));
@@ -2054,6 +2173,9 @@ bool BDS_Mesh::collapse_edge(BDS_Edge * e, BDS_Point * p, const double eps)
         p->Y = Y;
         p->Z = Z;
         n2 = t->N();
+
+	if(n1*n2 < 0) return false;
+	
         double s_before = surface_triangle(pts[0], pts[1], pts[2]);
         // normals should not be opposed or change too dramatically
         // this does not concern the triangles with the small edge that
@@ -2195,6 +2317,9 @@ bool BDS_Mesh::move_point(BDS_Point * p, double X, double Y, double Z)
     p->X = oldx;
     p->Y = oldy;
     p->Z = oldz;
+
+    if (n1*n2 < 0) return false;
+
     double angle = n1.angle(n2);
     if(fabs(angle) > M_PI / 3)
       return false;
@@ -2227,27 +2352,11 @@ void BDS_Mesh::snap_point(BDS_Point * p, BDS_Mesh * geom_mesh)
   else if(p->g && p->g->classif_degree == 2 && geom_mesh) {
     double xx, yy, zz;
     std::list < BDS_Triangle * >l;
-    BDS_GeomEntity *gg =
-      geom_mesh->get_geom(p->g->classif_tag, p->g->classif_degree);
-    gg->getClosestTriangles(p->X, p->Y, p->Z, l, p->radius_of_curvature, xx,
-                            yy, zz);
+    BDS_GeomEntity *gg = geom_mesh->get_geom(p->g->classif_tag, p->g->classif_degree);
+    gg->getClosestTriangles(p->X, p->Y, p->Z, l, p->radius_of_curvature, xx, yy, zz);
 
     bool ok = project_point_on_a_list_of_triangles(p, l, X, Y, Z);
 
-    /*
-       std::list<BDS_Triangle *>::iterator tit  = l.begin();  
-       std::list<BDS_Triangle *>::iterator tite =  l.end();
-       BDS_Vector n1 = p->N(), n2;      
-       while(tit!=tite){
-       n2 += (*tit)->N();
-       tit++;
-       }
-       n2 /= sqrt(n2*n2);
-       n1 /= sqrt(n1*n1);
-       double angle = n1.angle(n2);
-       if(fabs(angle) > M_PI/2 )ok= false;
-     */
-
     if(!ok) {
       if(move_point(p, xx, yy, zz)) {
         SNAP_SUCCESS++;
@@ -2275,6 +2384,9 @@ bool BDS_Mesh::smooth_point(BDS_Point * p, BDS_Mesh * geom_mesh)
 
   if(p->g && p->g->classif_degree <= 1)
     return false;
+  //  if(!p->g->surf)
+  //    return false;
+
 
   double X = 0;
   double Y = 0;
@@ -2302,6 +2414,7 @@ bool BDS_Mesh::smooth_point(BDS_Point * p, BDS_Mesh * geom_mesh)
 
 void BDS_Mesh::compute_metric_edge_lengths(const BDS_Metric & metric)
 {
+  
 
   // printf("computation of metric lengths\n");
   {
@@ -2313,6 +2426,14 @@ void BDS_Mesh::compute_metric_edge_lengths(const BDS_Metric & metric)
           metric.target_length(0.5 * ((*it)->p2->X + (*it)->p1->X),
                                0.5 * ((*it)->p2->Y + (*it)->p1->Y),
                                0.5 * ((*it)->p2->Z + (*it)->p1->Z));
+	BDS_Edge *e = (*it);
+	BDS_GeomEntity *g = e->g;
+	if (g && g->classif_degree == 1)
+	  {
+	    double l = (*it)->length() * 4;
+	    if (l < (*it)->target_length)
+	      (*it)->target_length =l;
+	  }   
       }
       ++it;
     }
@@ -2324,7 +2445,11 @@ void BDS_Mesh::compute_metric_edge_lengths(const BDS_Metric & metric)
     while(it != ite) {
       BDS_Edge *e = (*it);
       BDS_GeomEntity *g = e->g;
-      if(g && g->surf) {
+     
+      // do not unrefined curves
+      // minimal length is the actual length 
+   
+        if(g && g->surf) {
         double curvature = g->surf->normalCurv(0.5 * (e->p1->X + e->p2->X),
                                                0.5 * (e->p1->Y + e->p2->Y),
                                                0.5 * (e->p1->Z + e->p2->Z));
@@ -2342,7 +2467,6 @@ void BDS_Mesh::compute_metric_edge_lengths(const BDS_Metric & metric)
           3.14159 * radius / metric.nb_elements_per_radius_of_curvature;
         e->target_length =
           metric.update_target_length(target, e->target_length);
-
       }
       ++it;
     }
@@ -2404,14 +2528,10 @@ int BDS_Mesh::adapt_mesh(const BDS_Metric & metric, bool smooth,
   SNAP_SUCCESS = 0;
   SNAP_FAILURE = 0;
 
-  // printf("METRIC %g %g %g\n",LC,metric._min,metric._max);
-
-  // add initial set of edges in a list
-
   std::list < BDS_Edge * >small_to_long(edges);
 
   // split edges
-  {
+  for(int ii=0;ii<2;ii++){
     std::list < BDS_Edge * >::iterator it = small_to_long.begin();
     std::list < BDS_Edge * >::iterator ite = small_to_long.end();
     compute_metric_edge_lengths(metric);
@@ -2421,45 +2541,17 @@ int BDS_Mesh::adapt_mesh(const BDS_Metric & metric, bool smooth,
         double length = (*it)->length();
         if(!(*it)->deleted && length > (*it)->target_length / 0.7) {
           split_edge(*it, 0.5, geom_mesh);
-          //split_edge (*it, 0.5, 0  );
           nb_modif++;
         }
       }
       ++it;
     }
+    cleanup();
+    small_to_long = edges;
   }
-
-  // re-create small_to_long
-  cleanup();
-  small_to_long = edges;
-
-  // split edges
-  {
-    std::list<BDS_Edge*>::iterator it = small_to_long.begin();
-    std::list<BDS_Edge*>::iterator ite  = small_to_long.end();
-    compute_metric_edge_lengths (metric);
-    
-    while (it != ite)
-      {
-	if ((*it)->numfaces()==2)
-	  {
-	    double length = (*it)->length();
-	    if (!(*it)->deleted && length > (*it)->target_length / 0.7 ){
-	      split_edge (*it, 0.5,geom_mesh );
-	      //split_edge (*it, 0.5, 0  );
-	      nb_modif++;
-	    }
-	  }
-	++it;
-      }
-    }
-  
-  // re-create small_to_long
-  cleanup();    
-  small_to_long = edges;
   
   // collapse 
-  {    	
+  for(int ii=0;ii<1;ii++){    	
     std::list<BDS_Edge*>::iterator it = small_to_long.begin();
     std::list<BDS_Edge*>::iterator ite  = small_to_long.end();
     //	compute_metric_edge_lengths (metric);
@@ -2481,9 +2573,9 @@ int BDS_Mesh::adapt_mesh(const BDS_Metric & metric, bool smooth,
 	}
 	++it;
       }
+    cleanup();  
+    small_to_long = edges;
   }
-  cleanup();  
-  small_to_long = edges;
   
   {    
     std::list<BDS_Edge*>::iterator it = small_to_long.begin();
@@ -2495,46 +2587,24 @@ int BDS_Mesh::adapt_mesh(const BDS_Metric & metric, bool smooth,
 	    BDS_Point *op[2];
 	    (*it)->oppositeof (op);
 	    
-	    double a1 = surface_triangle ( (*it)->p1 , (*it)->p2 , op[0] );
-	    double a2 = surface_triangle ( (*it)->p1 , (*it)->p2 , op[1] );
-	    double b1 = surface_triangle ( (*it)->p1 , op[0] , op[1] );
-	    double b2 = surface_triangle ( (*it)->p2 , op[0] , op[1] );
-	    
-	    double cb1[3],cb2[3];
-	    normal_triangle ( (*it)->p1 , op[0] , op[1],cb1 );
-	    normal_triangle ( (*it)->p2 , op[0] , op[1],cb2 );
-	    
-	    double prosc = cb1[0]*cb2[0]+cb1[1]*cb2[1]+cb1[2]*cb2[2];
-	    
-	    if (fabs(a1+a2-b1-b2) < 0.1 * (a1+a2))
+	    double qa1 = quality_triangle ( (*it)->p1 , (*it)->p2 , op[0] );
+	    double qa2 = quality_triangle ( (*it)->p1 , (*it)->p2 , op[1] );
+	    double qb1 = quality_triangle ( (*it)->p1 , op[0] , op[1] );
+	    double qb2 = quality_triangle ( (*it)->p2 , op[0] , op[1] );
+	    double qa = (qa1<qa2)?qa1:qa2; 
+	    double qb = (qb1<qb2)?qb1:qb2; 
+	    if (qb > qa)	      
 	      {
-		double qa1 = quality_triangle ( (*it)->p1 , (*it)->p2 , op[0] );
-		double qa2 = quality_triangle ( (*it)->p1 , (*it)->p2 , op[1] );
-		double qb1 = quality_triangle ( (*it)->p1 , op[0] , op[1] );
-		double qb2 = quality_triangle ( (*it)->p2 , op[0] , op[1] );
-		
-		double d = dist_droites_gauches((*it)->p1, (*it)->p2, 
-						op[0],op[1]);
-		BDS_Vector v1 (*((*it)->p1), *((*it)->p2));
-		BDS_Vector v2 (*(op[0]),*(op[1]));
-		
-		double dd = sqrt (v1*v1 + v2*v2);
-
-		double qa = (qa1<qa2)?qa1:qa2; 
-		double qb = (qb1<qb2)?qb1:qb2; 
-		//		  printf("qa %g qb %g ..\n",qa,qb);
-		if (qb > qa && d < 0.01 * dd)
-		  {
-		    nb_modif++;
-		    swap_edge ( *it );
-		  }
+		nb_modif++;
+		swap_edge ( *it );
 	      }
-	  }
+	  }	
 	++it;
       }
   }
+
   cleanup();  
-  if (smooth){
+  if (smooth ){
     Msg(INFO,"smoothing %d points\n",points.size());
     std::set<BDS_Point*, PointLessThan>::iterator it   = points.begin();
     std::set<BDS_Point*, PointLessThan>::iterator ite  = points.end();
