@@ -1,6 +1,6 @@
-// $Id: Generator.cpp,v 1.73 2005/10/26 15:19:24 geuzaine Exp $
+// $Id: Generator.cpp,v 1.81 2006/02/26 16:26:09 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "OpenFile.h"
 #include "Views.h"
 #include "PartitionMesh.h"
+#include "OS.h"
 
 extern Mesh *THEM;
 extern Context_T CTX;
@@ -174,12 +175,19 @@ void GetStatistics(double stat[50])
 
 }
 
+static double SumOfAllLc = 0.;
+void GetSumOfAllLc(void *a, void *b)
+{
+  Vertex *v = *(Vertex **) a;
+  SumOfAllLc += v->lc;
+}
+
 void ApplyLcFactor_Point(void *a, void *b)
 {
   Vertex *v = *(Vertex **) a;
   if(v->lc <= 0.0) {
-    Msg(GERROR,
-        "Wrong characteristic length (%g <= 0) for Point %d, defaulting to 1.0",
+    Msg(GERROR, 
+	"Wrong characteristic length (%g <= 0) for Point %d, defaulting to 1.0",
         v->lc, v->Num);
     v->lc = 1.0;
   }
@@ -232,35 +240,55 @@ void Move_SimplexBaseToSimplex(Mesh * M, int dimension)
   }
 }
 
+bool TooManyElements(Mesh *M, int dim){
+  if(CTX.expert_mode || !Tree_Nbr(M->Points)) return false;
+
+  // try to detect obvious mistakes in characteristic lenghts (one of
+  // the most common cause for erroneous bug reports on the mailing
+  // list)
+  SumOfAllLc = 0.;
+  Tree_Action(M->Points, GetSumOfAllLc);
+  SumOfAllLc /= (double)Tree_Nbr(M->Points);
+  if(pow(CTX.lc / SumOfAllLc, dim) < 1.e7) return false;
+
+  return !GetBinaryAnswer("Your choice of characteristic lengths will likely produce\n"
+			  "a very large mesh. Do you really want to continue?\n\n"
+			  "(To disable this warning in the future, select `Enable\n"
+			  "expert mode' in the option dialog.)",
+			  "Continue", "Cancel");
+}
+
 void Maillage_Dimension_1(Mesh * M)
 {
-  double t1, t2;
+  if(TooManyElements(M, 1)) return;
 
-  t1 = Cpu();
+  double t1 = Cpu();
 
   Tree_Action(M->Curves, Maillage_Curve);
 
-  t2 = Cpu();
+  double t2 = Cpu();
   M->timing[0] = t2 - t1;
 }
 
 void Maillage_Dimension_2(Mesh * M)
 {
-  int i;
-  Curve *c, *neew, C;
-  double t1, t2, shortest = 1.e300;
+  if(TooManyElements(M, 2)) return;
 
-  t1 = Cpu();
+  double shortest = 1.e300;
+
+  double t1 = Cpu();
 
   // create reverse 1D meshes
 
   List_T *Curves = Tree2List(M->Curves);
-  for(i = 0; i < List_Nbr(Curves); i++) {
+  for(int i = 0; i < List_Nbr(Curves); i++) {
+    Curve *c;
     List_Read(Curves, i, &c);
     if(c->Num > 0) {
       if(c->l < shortest)
         shortest = c->l;
-      neew = &C;
+      Curve C;
+      Curve *neew = &C;
       neew->Num = -c->Num;
       Tree_Query(M->Curves, &neew);
       neew->Vertices =
@@ -281,7 +309,7 @@ void Maillage_Dimension_2(Mesh * M)
   if(CTX.mesh.algo_recombine == 2)
     Recombine_All(M);
 
-  t2 = Cpu();
+  double t2 = Cpu();
 
   M->timing[1] = t2 - t1;
 }
@@ -300,16 +328,15 @@ void TransferData(void *a, void *b)
 
 void Maillage_Dimension_3(Mesh * M)
 {
-  Volume *v;
-  double t1, t2;
-  Volume *vol;
+  if(TooManyElements(M, 3)) return;
 
-  t1 = Cpu();
+  double t1 = Cpu();
 
   // merge all the delaunay parts in a single special volume
-  v = Create_Volume(99999, 99999);
+  Volume *v = Create_Volume(99999, 99999);
   List_T *list = Tree2List(M->Volumes);
   for(int i = 0; i < List_Nbr(list); i++) {
+    Volume *vol;
     List_Read(list, i, &vol);
     if((!vol->Extrude || !vol->Extrude->mesh.ExtrudeMesh) &&
        (vol->Method != TRANSFINI)) {
@@ -339,7 +366,7 @@ void Maillage_Dimension_3(Mesh * M)
 
   List_Delete(list);
 
-  t2 = Cpu();
+  double t2 = Cpu();
 
   M->timing[2] = t2 - t1;
 }
@@ -360,8 +387,6 @@ void Init_Mesh0(Mesh * M)
   M->PhysicalGroups = NULL;
   M->Partitions = NULL;
   M->Metric = NULL;
-  M->BGM.bgm = NULL;
-  M->Grid.init = 0;
 }
 
 void Init_Mesh(Mesh * M)
@@ -388,9 +413,9 @@ void Init_Mesh(Mesh * M)
   Tree_Action(M->Points, Free_Vertex);  
   Tree_Delete(M->Points);
 
-  // Note: don't free the simplices here (with 
-  // Tree_Action (M->Simplexes, Free_Simplex)): we free them 
-  // in each curve, surface, volume
+  // Note: don't free the simplices here (with Tree_Action
+  // (M->Simplexes, Free_Simplex)): we free them in each curve,
+  // surface, volume
   Tree_Delete(M->Simplexes);
 
   Tree_Action(M->Curves, Free_Curve);
@@ -417,10 +442,8 @@ void Init_Mesh(Mesh * M)
   if(M->Metric)
     delete M->Metric;
 
-  List_Delete(M->BGM.bgm);
-
-  if(M->Grid.init)
-    List_Delete(M->Grid.Bricks);
+  if(M->normals)
+    delete M->normals;
 
   M->Vertices = Tree_Create(sizeof(Vertex *), compareVertex);
   M->Simplexes = Tree_Create(sizeof(Simplex *), compareSimplex);
@@ -433,12 +456,10 @@ void Init_Mesh(Mesh * M)
   M->PhysicalGroups = List_Create(5, 5, sizeof(PhysicalGroup *));
   M->Partitions = List_Create(5, 5, sizeof(MeshPartition *));
   M->Metric = new GMSHMetric;
-  M->BGM.bgm = NULL;
-  M->Grid.init = 0;
+  M->normals = new smooth_normals(CTX.mesh.angle_smooth_normals);
 
   M->status = 0;
-
-  Create_BgMesh(WITHPOINTS, .2, M);
+  M->BackgroundMeshType = WITHPOINTS;
 
   for(int i = 0; i < 3; i++){
     M->timing[i] = 0.0;
