@@ -1,7 +1,7 @@
 #ifndef _MESH_H_
 #define _MESH_H_
 
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -76,18 +76,13 @@
 #define EXTERN      1
 #define INTERN      2
 
-#define CONSTANT    1
 #define ONFILE      2
 #define WITHPOINTS  3
-#define FUNCTION    4
 
 #define TRANSFINI 1
 #define LIBRE     2
 #define ELLIPTIC  3
 
-#define BOULE 1
-#define BOITE 2
-
 #define NB_HISTOGRAM 100
 
 class BDS_Mesh;
@@ -196,18 +191,6 @@ class NXE{
   NXE();
 };
 
-typedef struct{
-  int N;
-  List_T *pT;
-}Brick;
-
-typedef struct{
-  int init;
-  List_T *Bricks;
-  Coord min, max;
-  int Nx, Ny, Nz;
-}Grid_T;
-
 typedef struct{
   double zaxis[3];
   double radius1;
@@ -228,8 +211,7 @@ struct _Surf{
   char Visible;
   int Method;
   int Recombine;
-// -1 is left, +1 is right, 0 is alternated
-  int Recombine_Dir; 
+  int Recombine_Dir; // -1 is left, +1 is right, 0 is alternated
   double RecombineAngle;
   int ipar[5];
   int Nu, Nv;
@@ -250,13 +232,10 @@ struct _Surf{
   float *ku, *kv, *cp;
   struct _Surf *Support;
   CylParam Cyl;
-  Grid_T Grid;  // fast search grid
   ExtrudeParams *Extrude;
-  //  BDS_Mesh *bds;
   DrawingColor Color;
   VertexArray *TriVertexArray;
   VertexArray *QuadVertexArray;
-  smooth_normals *normals;
 };
 
 typedef struct _Surf Surface;
@@ -342,12 +321,6 @@ typedef struct{
 
 typedef struct _Mesh Mesh;
 
-typedef struct{
-  int Typ;
-  double lc;
-  List_T *bgm;
-}LcField;
-
 typedef struct{
   double t1, t2, f1, f2, incl;
   Vertex *v[4];
@@ -375,7 +348,6 @@ typedef struct{
   CircParam Circle;
   char functu[256], functv[256], functw[256];
   DrawingColor Color;
-  //  BDS_Mesh *bds;
   VertexArray *LinVertexArray;
 }Curve;
 
@@ -412,8 +384,8 @@ struct _Mesh{
   Tree_T *EdgeLoops;
   List_T *PhysicalGroups;
   List_T *Partitions;
-  Grid_T Grid; // fast search grid
-  LcField BGM; // background mesh
+  int BackgroundMeshType;
+  smooth_normals *normals; // container for smooth normals
   double timing[3]; // timing for 1d, 2d and 3d mesh
   double quality_gamma[3]; // mesh quality statistics
   double quality_eta[3]; // mesh quality statistics
@@ -444,11 +416,11 @@ void mai3d(Mesh *M, int Asked);
 
 void Init_Mesh0(Mesh *M);
 void Init_Mesh(Mesh *M);
-void Create_BgMesh(int i, double d, Mesh *m);
 void Print_Geo(Mesh *M, char *c);
 void Print_Mesh(Mesh *M, char *c, int Type);
 void Read_Mesh(Mesh *M, FILE *fp, char *filename, int Type);
 void GetStatistics(double s[50]);
+void GetDefaultMeshFileName(Mesh *M, int Type, char *name);
 
 void Maillage_Dimension_1(Mesh *M);
 void Maillage_Dimension_2(Mesh *M);
@@ -494,14 +466,14 @@ void ReOrientSurfaceMesh(Surface *s);
 
 void Move_SimplexBaseToSimplex(Mesh * M, int dimension);
 
-double Lc_XYZ(double X, double Y, double Z, Mesh *m);
+double BGMXYZ(double X, double Y, double Z);
 void ActionLiss(void *data, void *dummy);
 void ActionLissSurf(void *data, void *dummy);
 int Recombine(Tree_T *TreeAllVert, Tree_T *TreeAllSimp, Tree_T *TreeAllQuad,
 		double a);
 int Recombine_All(Mesh *M);
 void ApplyLcFactor(Mesh *M);
-void ExportMeshStatistics(Mesh *M, char *filename, int volume=1, int surface=1);
+void ExportMeshStatistics(Mesh *M, char *filename, int vol=1, int surf=1, int lin=1);
 
 void Degre1();
 void Degre2(int dim);
