@@ -1,6 +1,6 @@
-// $Id: Nurbs.cpp,v 1.16 2005/01/01 19:35:31 geuzaine Exp $
+// $Id: Nurbs.cpp,v 1.18 2006/01/07 16:12:32 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -119,6 +119,8 @@ Vertex InterpolateUBS(Curve * Curve, double u, int derivee)
 
   if(iCurve > NbCurves)
     iCurve = NbCurves;
+  else if (iCurve < 1)
+    iCurve = 1;
 
   t1 = (double)(iCurve - 1) / (double)(NbCurves);
   t2 = (double)(iCurve) / (double)(NbCurves);
