@@ -1,6 +1,6 @@
-// $Id: Read_Mesh.cpp,v 1.95 2005/10/16 03:15:31 geuzaine Exp $
+// $Id: Read_Mesh.cpp,v 1.98.2.4 2006/04/16 18:56:10 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,13 +19,14 @@
 // 
 // Please report all bugs and problems to <gmsh@geuz.org>.
 
+#include <map>
+#include <vector>
 #include "Gmsh.h"
 #include "Geo.h"
 #include "CAD.h"
 #include "Mesh.h"
 #include "3D_Mesh.h"
 #include "Create.h"
-#include "MinMax.h"
 #include "Numeric.h"
 #include "Context.h"
 #include "PartitionMesh.h"
@@ -61,10 +62,85 @@ Curve *addElementaryCurve(Mesh * M, int Num)
   if(!(c = FindCurve(Num, M))) {
     c = Create_Curve(Num, MSH_SEGM_DISCRETE, 0, NULL, NULL, -1, -1, 0., 1.);
     Tree_Add(M->Curves, &c);
+    CreateReversedCurve(M, c);
   }
   return c;
 }
 
+class nodex{
+ public:
+  Vertex *v;
+  int left, right; // index left/right segment in simplex list
+  nodex(Vertex *vv) : v(vv), left(-1), right(-1) {}
+};
+
+void addVerticesInCurve(void *a, void *b)
+{
+  Curve *c = *(Curve**)a;
+
+  // check if already done or if nothing to do
+  if(List_Nbr(c->Vertices) || !Tree_Nbr(c->SimplexesBase)) 
+    return;
+
+  List_T *elm = Tree2List(c->SimplexesBase);
+
+  // construct node x segment connectivity
+  std::map<int, nodex*> nodes;
+  for(int i = 0; i < List_Nbr(elm); i++){
+    SimplexBase *s = *(SimplexBase**)List_Pointer(elm, i);
+    for(int j = 0; j < 2; j++){
+      if(!nodes.count(s->V[j]->Num))
+	nodes[s->V[j]->Num] = new nodex(s->V[j]);
+      if(!j)
+	nodes[s->V[j]->Num]->right = i;
+      else
+	nodes[s->V[j]->Num]->left = i;
+    }
+  }
+
+  // find starting element (or use the first one for closed curve)
+  std::map<int, nodex*>::const_iterator beg = nodes.begin();
+  std::map<int, nodex*>::const_iterator end = nodes.end();
+  int start = 0;
+  while(beg != end){
+    if((*beg).second->left < 0){ // no element to the left
+      start = (*beg).second->right;
+      break;
+    }
+    beg++;
+  }
+
+  // add nodes by following the line segments
+  if(start < 0){
+    Msg(GERROR, "Something is wrong in mesh of curve %d", c->Num);
+  }
+  else{
+    int N = List_Nbr(elm);
+    SimplexBase *s = *(SimplexBase**)List_Pointer(elm, start);
+    List_Add(c->Vertices, &s->V[0]);
+    while(N > 0){
+      if(s->VSUP) List_Add(c->Vertices, &s->VSUP[0]);
+      List_Add(c->Vertices, &s->V[1]);
+      nodex *n = nodes[s->V[1]->Num];
+      if(n->left != start){
+	Msg(GERROR, "Wrong orientation of element %d in curve %d", s->Num, c->Num);
+	break;
+      }
+      start = n->right;
+      if(start >= 0){
+	s = *(SimplexBase**)List_Pointer(elm, start);
+      }
+      else if(N != 1){
+	Msg(GERROR, "Something is wrong in mesh of curve %d", c->Num);
+	break;
+      }
+      N--;
+    }
+  }
+  
+  List_Delete(elm);
+}
+
 Surface *addElementarySurface(Mesh * M, int Num)
 {
   Surface *s;
@@ -146,21 +222,28 @@ int getNbrNodes(int Type)
   }
 }
 
-double SetLC(Vertex *v1, Vertex *v2, Vertex *v3, Vertex *v4 = 0)
+double SetLC(Vertex *v1, Vertex *v2, Vertex *v3 = 0, Vertex *v4 = 0)
 { 
   double lc1 = sqrt((v1->Pos.X - v2->Pos.X) * (v1->Pos.X - v2->Pos.X) +
 		    (v1->Pos.Y - v2->Pos.Y) * (v1->Pos.Y - v2->Pos.Y) +
 		    (v1->Pos.Z - v2->Pos.Z) * (v1->Pos.Z - v2->Pos.Z));
-  double lc2 = sqrt((v1->Pos.X - v3->Pos.X) * (v1->Pos.X - v3->Pos.X) +
-		    (v1->Pos.Y - v3->Pos.Y) * (v1->Pos.Y - v3->Pos.Y) +
-		    (v1->Pos.Z - v3->Pos.Z) * (v1->Pos.Z - v3->Pos.Z));
-  double lc3 = sqrt((v2->Pos.X - v3->Pos.X) * (v2->Pos.X - v3->Pos.X) +
-		    (v2->Pos.Y - v3->Pos.Y) * (v2->Pos.Y - v3->Pos.Y) +
-		    (v2->Pos.Z - v3->Pos.Z) * (v2->Pos.Z - v3->Pos.Z));
-  double lc = DMAX(lc1, DMAX(lc2, lc3)) * CTX.mesh.lc_factor;
-  v1->lc = v2->lc = v3->lc = lc;
-  if(v4) v4->lc = lc;
-  return lc;
+  if(!v3){
+    double lc = lc1 * CTX.mesh.lc_factor;
+    v1->lc = v2->lc = lc;
+    return lc;
+  }
+  else{
+    double lc2 = sqrt((v1->Pos.X - v3->Pos.X) * (v1->Pos.X - v3->Pos.X) +
+		      (v1->Pos.Y - v3->Pos.Y) * (v1->Pos.Y - v3->Pos.Y) +
+		      (v1->Pos.Z - v3->Pos.Z) * (v1->Pos.Z - v3->Pos.Z));
+    double lc3 = sqrt((v2->Pos.X - v3->Pos.X) * (v2->Pos.X - v3->Pos.X) +
+		      (v2->Pos.Y - v3->Pos.Y) * (v2->Pos.Y - v3->Pos.Y) +
+		      (v2->Pos.Z - v3->Pos.Z) * (v2->Pos.Z - v3->Pos.Z));
+    double lc = DMAX(lc1, DMAX(lc2, lc3)) * CTX.mesh.lc_factor;
+    v1->lc = v2->lc = v3->lc = lc;
+    if(v4) v4->lc = lc;
+    return lc;
+  }
 }
 
 void Read_Mesh_MSH(Mesh * M, FILE * fp)
@@ -186,7 +269,8 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 
   while(1) {
     do {
-      fgets(String, sizeof(String), fp);
+      if(!fgets(String, sizeof(String), fp))
+	break;
       if(feof(fp))
         break;
     } while(String[0] != '$');
@@ -359,6 +443,7 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
           simp->Num = Num;
           simp->iEnt = Elementary;
           simp->iPart = addMeshPartition(Partition, M);
+	  SetLC(vertsp[0], vertsp[1]);
 	  if(Type == LGN2){
 	    simp->VSUP = (Vertex **) Malloc(1 * sizeof(Vertex *));
 	    simp->VSUP[0] = vertsp[2];
@@ -368,11 +453,11 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
 	    Msg(GERROR, "Line element %d already exists", simp->Num);
 	    Free_SimplexBase(&simp, 0);
 	  }
-	  else{
-	    // this can be quite slow if there are many nodes on the curve...
-	    for(i = 0; i < Nbr_Nodes; i++)
-	      List_Insert(c->Vertices, &vertsp[i], fcmp_int);
-	  }
+	  // we don't insert the vertices in the list of vertices at
+	  // this point, since we need the list to ordered (and
+	  // consistent!), and simply doing a List_Insert and sorting
+	  // according to node numbers is not enough (in addition to
+	  // being slow): see addVerticesInCurve() below.
           break;
         case TRI1:
         case TRI2:
@@ -560,7 +645,8 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
     }
 
     do {
-      fgets(String, sizeof(String), fp);
+      if(!fgets(String, sizeof(String), fp))
+	Msg(GERROR, "Prematured end of mesh file");
       if(feof(fp))
         Msg(GERROR, "Prematured end of mesh file");
     } while(String[0] != '$');
@@ -578,6 +664,9 @@ void Read_Mesh_MSH(Mesh * M, FILE * fp)
   else
     M->status = -1;
 
+  // add vertices in curves
+  Tree_Action(M->Curves, addVerticesInCurve);
+
   // For efficiency reasons, we store the partition index (and not the
   // partition number) in the various mesh elements. We need to
   // re-sort the list according to these indices to allow direct
