@@ -1,6 +1,6 @@
-// $Id: Utils.cpp,v 1.28 2005/01/01 19:35:31 geuzaine Exp $
+// $Id: Utils.cpp,v 1.31 2006/01/28 21:13:35 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #else
 #define NRANSI
 #include "nrutil.h"
+void dsvdcmp(double **a, int m, int n, double w[], double **v);
 #endif
 
 extern Context_T CTX;
@@ -160,7 +161,9 @@ void MeanPlane(List_T * points, Surface * s)
     t2[2] = v3.Pos.Z - v1.Pos.Z;
     norme(t1);
     norme(t2);
-    prodve(t1, t2, res2);
+    // prodve(t1, t2, res2);
+    // Warning: the rest of the code assumes res = t2 x t1, not t1 x t2 (WTF?)
+    prodve(t2, t1, res2); 
     norme(res2);
     prodve(res, res2, c);
     prosca(res, res2, &cosc);
