@@ -1,6 +1,6 @@
-// $Id: Numeric.cpp,v 1.23 2005/03/11 05:47:56 geuzaine Exp $
+// $Id: Numeric.cpp,v 1.25 2006/01/17 17:09:05 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -45,7 +45,10 @@ int check_gsl()
 {
   // check version
   int major, minor;
-  sscanf(gsl_version, "%d.%d", &major, &minor);
+  if(!sscanf(gsl_version, "%d.%d", &major, &minor)){
+    Msg(FATAL, "Cannot retreive GSL version");
+    return 0;
+  }
   if(major < 1 || (major == 1 && minor < 2)) {
     Msg(FATAL1, "Your GSL version (%d.%d.X) has a bug in the singular value",
         major, minor);
