@@ -110,64 +110,21 @@ typedef union {
 #define	tGMSH_MAJOR_VERSION	357
 #define	tGMSH_MINOR_VERSION	358
 #define	tGMSH_PATCH_VERSION	359
-#define	tB_SPLINE_SURFACE_WITH_KNOTS	360
-#define	tB_SPLINE_CURVE_WITH_KNOTS	361
-#define	tCARTESIAN_POINT	362
-#define	tTRUE	363
-#define	tFALSE	364
-#define	tUNSPECIFIED	365
-#define	tU	366
-#define	tV	367
-#define	tEDGE_CURVE	368
-#define	tVERTEX_POINT	369
-#define	tORIENTED_EDGE	370
-#define	tPLANE	371
-#define	tFACE_OUTER_BOUND	372
-#define	tEDGE_LOOP	373
-#define	tADVANCED_FACE	374
-#define	tVECTOR	375
-#define	tDIRECTION	376
-#define	tAXIS2_PLACEMENT_3D	377
-#define	tISO	378
-#define	tENDISO	379
-#define	tENDSEC	380
-#define	tDATA	381
-#define	tHEADER	382
-#define	tFILE_DESCRIPTION	383
-#define	tFILE_SCHEMA	384
-#define	tFILE_NAME	385
-#define	tMANIFOLD_SOLID_BREP	386
-#define	tCLOSED_SHELL	387
-#define	tADVANCED_BREP_SHAPE_REPRESENTATION	388
-#define	tFACE_BOUND	389
-#define	tCYLINDRICAL_SURFACE	390
-#define	tCONICAL_SURFACE	391
-#define	tCIRCLE	392
-#define	tTRIMMED_CURVE	393
-#define	tGEOMETRIC_SET	394
-#define	tCOMPOSITE_CURVE_SEGMENT	395
-#define	tCONTINUOUS	396
-#define	tCOMPOSITE_CURVE	397
-#define	tTOROIDAL_SURFACE	398
-#define	tPRODUCT_DEFINITION	399
-#define	tPRODUCT_DEFINITION_SHAPE	400
-#define	tSHAPE_DEFINITION_REPRESENTATION	401
-#define	tELLIPSE	402
-#define	tAFFECTPLUS	403
-#define	tAFFECTMINUS	404
-#define	tAFFECTTIMES	405
-#define	tAFFECTDIVIDE	406
-#define	tOR	407
-#define	tAND	408
-#define	tEQUAL	409
-#define	tNOTEQUAL	410
-#define	tAPPROXEQUAL	411
-#define	tLESSOREQUAL	412
-#define	tGREATEROREQUAL	413
-#define	tCROSSPRODUCT	414
-#define	tPLUSPLUS	415
-#define	tMINUSMINUS	416
-#define	UNARYPREC	417
+#define	tAFFECTPLUS	360
+#define	tAFFECTMINUS	361
+#define	tAFFECTTIMES	362
+#define	tAFFECTDIVIDE	363
+#define	tOR	364
+#define	tAND	365
+#define	tEQUAL	366
+#define	tNOTEQUAL	367
+#define	tAPPROXEQUAL	368
+#define	tLESSOREQUAL	369
+#define	tGREATEROREQUAL	370
+#define	tCROSSPRODUCT	371
+#define	tPLUSPLUS	372
+#define	tMINUSMINUS	373
+#define	UNARYPREC	374
 
 
 extern YYSTYPE yylval;
