@@ -1,6 +1,6 @@
-// $Id: Evaluate.cpp,v 1.23 2005/10/28 17:25:59 geuzaine Exp $
+// $Id: Evaluate.cpp,v 1.25.2.1 2006/04/20 12:38:42 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -171,7 +171,8 @@ void GMSH_EvaluatePlugin::evaluate(Post_View *v1, List_T *list1, int nbElm1,
   v1->Changed = 1;
 
   int nb = List_Nbr(list1) / nbElm1;
-  for(int i = 0; i < List_Nbr(list1); i += nb) {
+  int nb2 = nbElm2 ? List_Nbr(list2) / nbElm2 : 0;
+  for(int i = 0, i2 = 0; i < List_Nbr(list1); i += nb, i2 += nb2) {
     double *x = (double *)List_Pointer_Fast(list1, i);
     double *y = (double *)List_Pointer_Fast(list1, i + nbNod);
     double *z = (double *)List_Pointer_Fast(list1, i + 2 * nbNod);
@@ -201,7 +202,7 @@ void GMSH_EvaluatePlugin::evaluate(Post_View *v1, List_T *list1, int nbElm1,
       }
       else{
 	val2 = (double *)List_Pointer_Fast(list2, 
-					   i + 3 * nbNod + 
+					   i2 + 3 * nbNod + 
 					   nbNod * nbComp * timeStep2 + nbComp * j);
 	for(int k = 0; k < nbComp; k++) w[k] = val2[k];
       }
@@ -317,16 +318,8 @@ Post_View *GMSH_EvaluatePlugin::execute(Post_View * v)
   if(_octree) delete _octree;
 
   // recompute min/max, etc.:
-  for(int i = 0; i < v1->NbTimeStep; i++){
-    v1->TimeStepMin[i] = VAL_INF;
-    v1->TimeStepMax[i] = -VAL_INF;
-  }
   v1->Min = VAL_INF;
   v1->Max = -VAL_INF;
-  for(int i = 0; i < 3; i++) {
-    v1->BBox[2 * i] = VAL_INF;
-    v1->BBox[2 * i + 1] = -VAL_INF;
-  }
   EndView(v1, 0, v1->FileName, v1->Name);
 
   return v1;
