@@ -1,6 +1,6 @@
-// $Id: Plugin.cpp,v 1.78 2005/07/11 16:22:35 geuzaine Exp $
+// $Id: Plugin.cpp,v 1.80.2.2 2006/04/15 17:27:04 geuzaine Exp $
 //
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -40,6 +40,8 @@
 #include "CutSphere.h"
 #include "Skin.h"
 #include "Extract.h"
+#include "ExtractElements.h"
+#include "ExtractEdges.h"
 #include "HarmonicToTime.h"
 #include "ModulusPhase.h"
 #include "Integrate.h"
@@ -52,8 +54,7 @@
 #include "Smooth.h"
 #include "Transform.h"
 #include "Triangulate.h"
-#include "SphericalRaise.h"
-#include "DisplacementRaise.h"
+#include "Warp.h"
 #include "StructuralSolver.h"
 #include "Eigenvectors.h"
 #include "Eigenvalues.h"
@@ -188,6 +189,10 @@ void GMSH_PluginManager::registerDefaultPlugins()
 		      ("Skin", GMSH_RegisterSkinPlugin()));
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
 		      ("Extract", GMSH_RegisterExtractPlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("ExtractElements", GMSH_RegisterExtractElementsPlugin()));
+    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
+		      ("ExtractEdges", GMSH_RegisterExtractEdgesPlugin()));
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
 		      ("DecomposeInSimplex", GMSH_RegisterDecomposeInSimplexPlugin()));
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
@@ -195,9 +200,7 @@ void GMSH_PluginManager::registerDefaultPlugins()
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
 		      ("Transform", GMSH_RegisterTransformPlugin()));
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
-		      ("SphericalRaise", GMSH_RegisterSphericalRaisePlugin()));
-    allPlugins.insert(std::pair < char *, GMSH_Plugin * >
-		      ("DisplacementRaise", GMSH_RegisterDisplacementRaisePlugin()));
+		      ("Warp", GMSH_RegisterWarpPlugin()));
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
 		      ("HarmonicToTime", GMSH_RegisterHarmonicToTimePlugin()));
     allPlugins.insert(std::pair < char *, GMSH_Plugin * >
