@@ -1,7 +1,7 @@
 #ifndef _GMSH_CLIENT_H_
 #define _GMSH_CLIENT_H_
 
-// Copyright (C) 1997-2005 C. Geuzaine, J.-F. Remacle
+// Copyright (C) 1997-2006 C. Geuzaine, J.-F. Remacle
 //
 // Permission is hereby granted, free of charge, to any person
 // obtaining a copy of this software and associated documentation
@@ -36,35 +36,37 @@
 
 #if !defined(WIN32) || defined(__CYGWIN__)
 
+#include <unistd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/un.h>
 #include <sys/time.h>
-#include <unistd.h>
 #include <netinet/in.h>
 #include <netdb.h>
 
 #else // pure windows
 
-#include <winsock2.h>
+#include <winsock.h>
+#include <process.h>
 
 #endif
 
 class GmshClient {
  private:
-  typedef enum{ CLIENT_START    = 1,
-		CLIENT_STOP     = 2,
-		CLIENT_INFO     = 10,
-		CLIENT_WARNING  = 11,
-		CLIENT_ERROR    = 12,
-		CLIENT_PROGRESS = 13,
-		CLIENT_VIEW     = 20,
-		CLIENT_OPTION_1 = 100,
-		CLIENT_OPTION_2 = 101,
-		CLIENT_OPTION_3 = 102,
-		CLIENT_OPTION_4 = 103,
-		CLIENT_OPTION_5 = 104 } MessageType;
+  typedef enum{ CLIENT_START        = 1,
+		CLIENT_STOP         = 2,
+		CLIENT_INFO         = 10,
+		CLIENT_WARNING      = 11,
+		CLIENT_ERROR        = 12,
+		CLIENT_PROGRESS     = 13,
+		CLIENT_MERGE_FILE   = 20, // old name was CLIENT_VIEW
+		CLIENT_PARSE_STRING = 21,
+		CLIENT_OPTION_1     = 100,
+		CLIENT_OPTION_2     = 101,
+		CLIENT_OPTION_3     = 102,
+		CLIENT_OPTION_4     = 103,
+		CLIENT_OPTION_5     = 104 } MessageType;
   int _sock;
   void _SendData(void *buffer, int bytes)
   {
@@ -84,24 +86,12 @@ class GmshClient {
     _SendData(&len, sizeof(int));
     _SendData(str, len);
   }
-  long _GetTime()
+  void _Idle(int ms)
   {
 #if !defined(WIN32) || defined(__CYGWIN__)
-    struct timeval tp;
-    gettimeofday(&tp, (struct timezone *)0);
-    return (long)tp.tv_sec * 1000000 + (long)tp.tv_usec;
+    usleep(1000 * ms);
 #else
-    return 0;
-#endif
-  }
-  void _Idle(double delay)
-  {
-#if !defined(WIN32) || defined(__CYGWIN__)
-    long t1 = _GetTime();
-    while(1) {
-      if(_GetTime() - t1 > 1.e6 * delay)
-	break;
-    }
+    Sleep(ms);
 #endif
   }
  public:
@@ -109,9 +99,16 @@ class GmshClient {
   ~GmshClient(){}
   int Connect(char *sockname)
   {
+#if defined(WIN32) && !defined(__CYGWIN__)
+    WSADATA wsaData;
+    int iResult = WSAStartup(MAKEWORD(2,2), &wsaData);
+    if(iResult != NO_ERROR)
+      return -4;  // Error: Couldn't initialize Windows sockets
+#endif
+
     // slight delay to be sure that the socket is bound by the
     // server before we attempt to connect to it...
-    _Idle(0.1);
+    _Idle(100);
 
     int portno;
     char remote[256];
@@ -132,18 +129,24 @@ class GmshClient {
     // create socket
     
     if(portno < 0){
+#if !defined(WIN32) || defined(__CYGWIN__)
       _sock = socket(PF_UNIX, SOCK_STREAM, 0);
       if(_sock < 0)
 	return -1;  // Error: Couldn't create socket
       // try to connect socket to given name
       struct sockaddr_un addr_un;
-      strcpy(addr_un.sun_path, sockname);
+      memset((char *) &addr_un, 0, sizeof(addr_un));
       addr_un.sun_family = AF_UNIX;
+      strcpy(addr_un.sun_path, sockname);
       for(int tries = 0; tries < 5; tries++) {
 	if(connect(_sock, (struct sockaddr *)&addr_un, sizeof(addr_un)) >= 0)
 	  return _sock;
-	_Idle(0.1);
+	_Idle(100);
       }
+#else
+      // Unix sockets are not available on Windows without Cygwin
+      return -1;
+#endif
     }
     else{
       _sock = socket(AF_INET, SOCK_STREAM, 0);
@@ -158,11 +161,10 @@ class GmshClient {
       addr_in.sin_family = AF_INET;
       memcpy((char *)&addr_in.sin_addr.s_addr, (char *)server->h_addr, server->h_length);
       addr_in.sin_port = htons(portno);
-      addr_in.sin_addr.s_addr = INADDR_ANY;
       for(int tries = 0; tries < 5; tries++) {
 	if(connect(_sock, (struct sockaddr *)&addr_in, sizeof(addr_in)) >= 0)
 	  return _sock;
-	_Idle(0.1);
+	_Idle(100);
       }
     }
     return -2; // Error: Couldn't connect
@@ -170,7 +172,11 @@ class GmshClient {
   void Start()
   {
     char tmp[256];
+#if !defined(WIN32) || defined(__CYGWIN__)
     sprintf(tmp, "%d", getpid());
+#else
+    sprintf(tmp, "%d", _getpid());
+#endif
     _SendString(CLIENT_START, tmp);
   }
   void Stop()
@@ -193,9 +199,18 @@ class GmshClient {
   {
     _SendString(CLIENT_PROGRESS, str);
   }
-  void View(char *str)
+  void View(char *str) 
   {
-    _SendString(CLIENT_VIEW, str);
+    // deprecated: use MergeFile(str) instead
+    _SendString(CLIENT_MERGE_FILE, str);
+  }
+  void MergeFile(char *str)
+  {
+    _SendString(CLIENT_MERGE_FILE, str);
+  }
+  void ParseString(char *str)
+  {
+    _SendString(CLIENT_PARSE_STRING, str);
   }
   void Option(int num, char *str)
   {
@@ -205,7 +220,12 @@ class GmshClient {
   }
   void Disconnect()
   {
+#if !defined(WIN32) || defined(__CYGWIN__)
     close(_sock);
+#else
+    closesocket(_sock);
+    WSACleanup();
+#endif
   }
 };
 
